@@ -1,8 +1,8 @@
-#!/bin/sh
+#!/bin/bash
 
 set -e -x
 
-GO_VER=${GO_VER:-1.5}
+GO_VER=${GO_VER:-1.4.3}
 
 docker run -it -v "${GOPATH}":/gopath -v "$(pwd)":/app -e "GOPATH=/gopath" -w /app golang:$GO_VER sh -c 'CGO_ENABLED=0 go build -a --installsuffix cgo --ldflags="-s" -o influxd ./cmd/influxd'
 
