@@ -28,14 +28,13 @@ func (cmd *Command) Run(args ...string) error {
 const usage = `
 Configure and start an InfluxDB server.
 
-Usage:
-
-	influxd [[command] [arguments]]
+Usage: influxd [[command] [arguments]]
 
 The commands are:
 
     backup               downloads a snapshot of a data node and saves it to disk
     config               display the default configuration
+    help                 display this help message
     restore              uses a snapshot of a data node to rebuild a cluster
     run                  run node with existing configuration
     version              displays the InfluxDB version
