@@ -12,6 +12,11 @@ var (
 
 	// ErrFieldTypeConflict is returned when a new field already exists with a different type.
 	ErrFieldTypeConflict = errors.New("field type conflict")
+
+	// ErrUpgradeEngine will be returned when it's determined that
+	// the server has encountered shards that are not in the `tsm1`
+	// format.
+	ErrUpgradeEngine = errors.New("\n\n" + upgradeMessage + "\n\n")
 )
 
 // ErrDatabaseNotFound indicates that a database operation failed on the
@@ -43,3 +48,13 @@ func IsClientError(err error) bool {
 
 	return false
 }
+
+const upgradeMessage = `*******************************************************************
+                 UNSUPPORTED SHARD FORMAT DETECTED
+
+As of version 0.11, only tsm shards are supported. Please use the
+influx_tsm tool to convert non-tsm shards.
+
+More information can be found at the documentation site:
+https://docs.influxdata.com/influxdb/v0.10/administration/upgrading
+*******************************************************************`
