@@ -10,6 +10,14 @@ import (
 	"github.com/influxdata/influxdb/influxql"
 )
 
+func BenchmarkQuery_String(b *testing.B) {
+	p := influxql.NewParser(strings.NewReader(`SELECT foo AS zoo, a AS b FROM bar WHERE value > 10 AND q = 'hello'`))
+	q, _ := p.ParseStatement()
+	for i := 0; i < b.N; i++ {
+		_ = q.String()
+	}
+}
+
 // Ensure a value's data type can be retrieved.
 func TestInspectDataType(t *testing.T) {
 	for i, tt := range []struct {
@@ -44,6 +52,7 @@ func TestDataType_String(t *testing.T) {
 		{influxql.String, "string"},
 		{influxql.Time, "time"},
 		{influxql.Duration, "duration"},
+		{influxql.Tag, "tag"},
 		{influxql.Unknown, "unknown"},
 	} {
 		if v := tt.typ.String(); tt.v != v {
@@ -334,8 +343,8 @@ func TestSelectStatement_HasWildcard(t *testing.T) {
 	}
 }
 
-// Test SELECT statement wildcard rewrite.
-func TestSelectStatement_RewriteWildcards(t *testing.T) {
+// Test SELECT statement field rewrite.
+func TestSelectStatement_RewriteFields(t *testing.T) {
 	var tests = []struct {
 		stmt    string
 		rewrite string
@@ -349,7 +358,7 @@ func TestSelectStatement_RewriteWildcards(t *testing.T) {
 		// Query wildcard
 		{
 			stmt:    `SELECT * FROM cpu`,
-			rewrite: `SELECT host, region, value1, value2 FROM cpu`,
+			rewrite: `SELECT host::tag, region::tag, value1::float, value2::integer FROM cpu`,
 		},
 
 		// Parser fundamentally prohibits multiple query sources
@@ -357,19 +366,19 @@ func TestSelectStatement_RewriteWildcards(t *testing.T) {
 		// Query wildcard with explicit
 		{
 			stmt:    `SELECT *,value1 FROM cpu`,
-			rewrite: `SELECT host, region, value1, value2, value1 FROM cpu`,
+			rewrite: `SELECT host::tag, region::tag, value1::float, value2::integer, value1::float FROM cpu`,
 		},
 
 		// Query multiple wildcards
 		{
 			stmt:    `SELECT *,* FROM cpu`,
-			rewrite: `SELECT host, region, value1, value2, host, region, value1, value2 FROM cpu`,
+			rewrite: `SELECT host::tag, region::tag, value1::float, value2::integer, host::tag, region::tag, value1::float, value2::integer FROM cpu`,
 		},
 
 		// Query wildcards with group by
 		{
 			stmt:    `SELECT * FROM cpu GROUP BY host`,
-			rewrite: `SELECT region, value1, value2 FROM cpu GROUP BY host`,
+			rewrite: `SELECT region::tag, value1::float, value2::integer FROM cpu GROUP BY host`,
 		},
 
 		// No GROUP BY wildcards
@@ -396,7 +405,7 @@ func TestSelectStatement_RewriteWildcards(t *testing.T) {
 			rewrite: `SELECT mean(value) FROM cpu WHERE time < now() GROUP BY host, region, time(1m)`,
 		},
 
-		// GROUP BY wildarde with fill
+		// GROUP BY wildcard with fill
 		{
 			stmt:    `SELECT mean(value) FROM cpu where time < now() GROUP BY *,time(1m) fill(0)`,
 			rewrite: `SELECT mean(value) FROM cpu WHERE time < now() GROUP BY host, region, time(1m) fill(0)`,
@@ -417,7 +426,40 @@ func TestSelectStatement_RewriteWildcards(t *testing.T) {
 		// Combo
 		{
 			stmt:    `SELECT * FROM cpu GROUP BY *`,
-			rewrite: `SELECT value1, value2 FROM cpu GROUP BY host, region`,
+			rewrite: `SELECT value1::float, value2::integer FROM cpu GROUP BY host, region`,
+		},
+
+		// Wildcard function with all fields.
+		{
+			stmt:    `SELECT mean(*) FROM cpu`,
+			rewrite: `SELECT mean(value1::float) AS mean_value1, mean(value2::integer) AS mean_value2 FROM cpu`,
+		},
+
+		{
+			stmt:    `SELECT distinct(*) FROM strings`,
+			rewrite: `SELECT distinct(string::string) AS distinct_string, distinct(value::float) AS distinct_value FROM strings`,
+		},
+
+		{
+			stmt:    `SELECT distinct(*) FROM bools`,
+			rewrite: `SELECT distinct(bool::boolean) AS distinct_bool, distinct(value::float) AS distinct_value FROM bools`,
+		},
+
+		// Wildcard function with some fields excluded.
+		{
+			stmt:    `SELECT mean(*) FROM strings`,
+			rewrite: `SELECT mean(value::float) AS mean_value FROM strings`,
+		},
+
+		{
+			stmt:    `SELECT mean(*) FROM bools`,
+			rewrite: `SELECT mean(value::float) AS mean_value FROM bools`,
+		},
+
+		// Wildcard function with an alias.
+		{
+			stmt:    `SELECT mean(*) AS alias FROM cpu`,
+			rewrite: `SELECT mean(value1::float) AS alias_value1, mean(value2::integer) AS alias_value2 FROM cpu`,
 		},
 	}
 
@@ -429,14 +471,31 @@ func TestSelectStatement_RewriteWildcards(t *testing.T) {
 		}
 
 		var ic IteratorCreator
-		ic.FieldDimensionsFn = func(sources influxql.Sources) (fields, dimensions map[string]struct{}, err error) {
-			fields = map[string]struct{}{"value1": struct{}{}, "value2": struct{}{}}
+		ic.FieldDimensionsFn = func(sources influxql.Sources) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error) {
+			source := sources[0].(*influxql.Measurement)
+			switch source.Name {
+			case "cpu":
+				fields = map[string]influxql.DataType{
+					"value1": influxql.Float,
+					"value2": influxql.Integer,
+				}
+			case "strings":
+				fields = map[string]influxql.DataType{
+					"value":  influxql.Float,
+					"string": influxql.String,
+				}
+			case "bools":
+				fields = map[string]influxql.DataType{
+					"value": influxql.Float,
+					"bool":  influxql.Boolean,
+				}
+			}
 			dimensions = map[string]struct{}{"host": struct{}{}, "region": struct{}{}}
 			return
 		}
 
 		// Rewrite statement.
-		rw, err := stmt.(*influxql.SelectStatement).RewriteWildcards(&ic)
+		rw, err := stmt.(*influxql.SelectStatement).RewriteFields(&ic)
 		if err != nil {
 			t.Errorf("%d. %q: error: %s", i, tt.stmt, err)
 		} else if rw == nil {
@@ -733,7 +792,7 @@ func TestTimeRange(t *testing.T) {
 
 		// Invalid time expressions.
 		{expr: `time > "2000-01-01 00:00:00"`, min: `0001-01-01T00:00:00Z`, max: `0001-01-01T00:00:00Z`, err: `invalid operation: time and *influxql.VarRef are not compatible`},
-		{expr: `time > '2262-04-11 13:47:17'`, min: `0001-01-01T00:00:00Z`, max: `0001-01-01T00:00:00Z`, err: `time 2262-04-11T13:47:17Z overflows time literal`},
+		{expr: `time > '2262-04-11 23:47:17'`, min: `0001-01-01T00:00:00Z`, max: `0001-01-01T00:00:00Z`, err: `time 2262-04-11T23:47:17Z overflows time literal`},
 		{expr: `time > '1677-09-20 19:12:43'`, min: `0001-01-01T00:00:00Z`, max: `0001-01-01T00:00:00Z`, err: `time 1677-09-20T19:12:43Z underflows time literal`},
 	} {
 		// Extract time range.
@@ -1032,10 +1091,13 @@ func TestReduce(t *testing.T) {
 		{in: `true <> false`, out: `true`},
 		{in: `true + false`, out: `true + false`},
 
-		// Time literals.
+		// Time literals with now().
 		{in: `now() + 2h`, out: `'2000-01-01T02:00:00Z'`, data: map[string]interface{}{"now()": now}},
 		{in: `now() / 2h`, out: `'2000-01-01T00:00:00Z' / 2h`, data: map[string]interface{}{"now()": now}},
 		{in: `4µ + now()`, out: `'2000-01-01T00:00:00.000004Z'`, data: map[string]interface{}{"now()": now}},
+		{in: `now() + 2000000000`, out: `'2000-01-01T00:00:02Z'`, data: map[string]interface{}{"now()": now}},
+		{in: `2000000000 + now()`, out: `'2000-01-01T00:00:02Z'`, data: map[string]interface{}{"now()": now}},
+		{in: `now() - 2000000000`, out: `'1999-12-31T23:59:58Z'`, data: map[string]interface{}{"now()": now}},
 		{in: `now() = now()`, out: `true`, data: map[string]interface{}{"now()": now}},
 		{in: `now() <> now()`, out: `false`, data: map[string]interface{}{"now()": now}},
 		{in: `now() < now() + 1h`, out: `true`, data: map[string]interface{}{"now()": now}},
@@ -1047,6 +1109,28 @@ func TestReduce(t *testing.T) {
 		{in: `now()`, out: `now()`},
 		{in: `946684800000000000 + 2h`, out: `'2000-01-01T02:00:00Z'`},
 
+		// Time literals.
+		{in: `'2000-01-01T00:00:00Z' + 2h`, out: `'2000-01-01T02:00:00Z'`},
+		{in: `'2000-01-01T00:00:00Z' / 2h`, out: `'2000-01-01T00:00:00Z' / 2h`},
+		{in: `4µ + '2000-01-01T00:00:00Z'`, out: `'2000-01-01T00:00:00.000004Z'`},
+		{in: `'2000-01-01T00:00:00Z' + 2000000000`, out: `'2000-01-01T00:00:02Z'`},
+		{in: `2000000000 + '2000-01-01T00:00:00Z'`, out: `'2000-01-01T00:00:02Z'`},
+		{in: `'2000-01-01T00:00:00Z' - 2000000000`, out: `'1999-12-31T23:59:58Z'`},
+		{in: `'2000-01-01T00:00:00Z' = '2000-01-01T00:00:00Z'`, out: `true`},
+		{in: `'2000-01-01T00:00:00.000000000Z' = '2000-01-01T00:00:00Z'`, out: `true`},
+		{in: `'2000-01-01T00:00:00Z' <> '2000-01-01T00:00:00Z'`, out: `false`},
+		{in: `'2000-01-01T00:00:00.000000000Z' <> '2000-01-01T00:00:00Z'`, out: `false`},
+		{in: `'2000-01-01T00:00:00Z' < '2000-01-01T00:00:00Z' + 1h`, out: `true`},
+		{in: `'2000-01-01T00:00:00.000000000Z' < '2000-01-01T00:00:00Z' + 1h`, out: `true`},
+		{in: `'2000-01-01T00:00:00Z' <= '2000-01-01T00:00:00Z' + 1h`, out: `true`},
+		{in: `'2000-01-01T00:00:00.000000000Z' <= '2000-01-01T00:00:00Z' + 1h`, out: `true`},
+		{in: `'2000-01-01T00:00:00Z' > '2000-01-01T00:00:00Z' - 1h`, out: `true`},
+		{in: `'2000-01-01T00:00:00.000000000Z' > '2000-01-01T00:00:00Z' - 1h`, out: `true`},
+		{in: `'2000-01-01T00:00:00Z' >= '2000-01-01T00:00:00Z' - 1h`, out: `true`},
+		{in: `'2000-01-01T00:00:00.000000000Z' >= '2000-01-01T00:00:00Z' - 1h`, out: `true`},
+		{in: `'2000-01-01T00:00:00Z' - ('2000-01-01T00:00:00Z' - 60s)`, out: `1m`},
+		{in: `'2000-01-01T00:00:00Z' AND '2000-01-01T00:00:00Z'`, out: `'2000-01-01T00:00:00Z' AND '2000-01-01T00:00:00Z'`},
+
 		// Duration literals.
 		{in: `10m + 1h - 60s`, out: `69m`},
 		{in: `(10m / 2) * 5`, out: `25m`},
@@ -1214,6 +1298,33 @@ func TestSelect_ColumnNames(t *testing.T) {
 	}
 }
 
+func TestSelect_Privileges(t *testing.T) {
+	stmt := &influxql.SelectStatement{
+		Target: &influxql.Target{
+			Measurement: &influxql.Measurement{Database: "db2"},
+		},
+		Sources: []influxql.Source{
+			&influxql.Measurement{Database: "db0"},
+			&influxql.Measurement{Database: "db1"},
+		},
+	}
+
+	exp := influxql.ExecutionPrivileges{
+		influxql.ExecutionPrivilege{Name: "db0", Privilege: influxql.ReadPrivilege},
+		influxql.ExecutionPrivilege{Name: "db1", Privilege: influxql.ReadPrivilege},
+		influxql.ExecutionPrivilege{Name: "db2", Privilege: influxql.WritePrivilege},
+	}
+
+	got, err := stmt.RequiredPrivileges()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if !reflect.DeepEqual(exp, got) {
+		t.Errorf("exp: %v, got: %v", exp, got)
+	}
+}
+
 func TestSources_Names(t *testing.T) {
 	sources := influxql.Sources([]influxql.Source{
 		&influxql.Measurement{
