@@ -659,6 +659,146 @@ func TestCallIterator_Last_Boolean(t *testing.T) {
 	}
 }
 
+// Ensure that a float iterator can be created for a mode() call.
+func TestCallIterator_Mode_Float(t *testing.T) {
+	itr, _ := influxql.NewModeIterator(&FloatIterator{Points: []influxql.FloatPoint{
+		{Time: 0, Value: 15, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 1, Value: 11, Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 1, Value: 10, Tags: ParseTags("region=us-west,host=hostA")},
+		{Time: 2, Value: 10, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 3, Value: 10, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 4, Value: 10, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 6, Value: 20, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 7, Value: 21, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 8, Value: 21, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 22, Value: 8, Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 23, Value: 8, Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 24, Value: 25, Tags: ParseTags("region=us-west,host=hostB")},
+	}},
+		influxql.IteratorOptions{
+			Expr:       MustParseExpr(`mode("value")`),
+			Dimensions: []string{"host"},
+			Interval:   influxql.Interval{Duration: 5 * time.Nanosecond},
+		},
+	)
+
+	if a, err := Iterators([]influxql.Iterator{itr}).ReadAll(); err != nil {
+		t.Fatalf("unexpected error: %s", err)
+	} else if !deep.Equal(a, [][]influxql.Point{
+		{&influxql.FloatPoint{Time: 0, Value: 10, Tags: ParseTags("host=hostA"), Aggregated: 0}},
+		{&influxql.FloatPoint{Time: 1, Value: 11, Tags: ParseTags("host=hostB"), Aggregated: 0}},
+		{&influxql.FloatPoint{Time: 5, Value: 21, Tags: ParseTags("host=hostA"), Aggregated: 0}},
+		{&influxql.FloatPoint{Time: 20, Value: 8, Tags: ParseTags("host=hostB"), Aggregated: 0}},
+	}) {
+		t.Fatalf("unexpected points: %s", spew.Sdump(a))
+	}
+}
+
+// Ensure that a integer iterator can be created for a mode() call.
+func TestCallIterator_Mode_Integer(t *testing.T) {
+	itr, _ := influxql.NewModeIterator(&IntegerIterator{Points: []influxql.IntegerPoint{
+		{Time: 0, Value: 15, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 1, Value: 11, Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 1, Value: 10, Tags: ParseTags("region=us-west,host=hostA")},
+		{Time: 2, Value: 10, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 3, Value: 10, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 4, Value: 10, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 6, Value: 20, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 7, Value: 21, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 8, Value: 21, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 22, Value: 8, Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 23, Value: 8, Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 24, Value: 25, Tags: ParseTags("region=us-west,host=hostB")},
+	}},
+		influxql.IteratorOptions{
+			Expr:       MustParseExpr(`mode("value")`),
+			Dimensions: []string{"host"},
+			Interval:   influxql.Interval{Duration: 5 * time.Nanosecond},
+		},
+	)
+
+	if a, err := Iterators([]influxql.Iterator{itr}).ReadAll(); err != nil {
+		t.Fatalf("unexpected error: %s", err)
+	} else if !deep.Equal(a, [][]influxql.Point{
+		{&influxql.IntegerPoint{Time: 0, Value: 10, Tags: ParseTags("host=hostA")}},
+		{&influxql.IntegerPoint{Time: 1, Value: 11, Tags: ParseTags("host=hostB")}},
+		{&influxql.IntegerPoint{Time: 5, Value: 21, Tags: ParseTags("host=hostA")}},
+		{&influxql.IntegerPoint{Time: 20, Value: 8, Tags: ParseTags("host=hostB")}},
+	}) {
+		t.Fatalf("unexpected points: %s", spew.Sdump(a))
+	}
+}
+
+// Ensure that a string iterator can be created for a mode() call.
+func TestCallIterator_Mode_String(t *testing.T) {
+	itr, _ := influxql.NewModeIterator(&StringIterator{Points: []influxql.StringPoint{
+		{Time: 0, Value: "15", Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 1, Value: "11", Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 1, Value: "10", Tags: ParseTags("region=us-west,host=hostA")},
+		{Time: 2, Value: "10", Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 3, Value: "10", Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 4, Value: "10", Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 6, Value: "20", Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 7, Value: "21", Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 7, Value: "21", Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 22, Value: "8", Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 23, Value: "8", Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 24, Value: "25", Tags: ParseTags("region=us-west,host=hostB")},
+	}},
+		influxql.IteratorOptions{
+			Expr:       MustParseExpr(`mode("value")`),
+			Dimensions: []string{"host"},
+			Interval:   influxql.Interval{Duration: 5 * time.Nanosecond},
+		},
+	)
+
+	if a, err := Iterators([]influxql.Iterator{itr}).ReadAll(); err != nil {
+		t.Fatalf("unexpected error: %s", err)
+	} else if !deep.Equal(a, [][]influxql.Point{
+		{&influxql.StringPoint{Time: 0, Value: "10", Tags: ParseTags("host=hostA")}},
+		{&influxql.StringPoint{Time: 1, Value: "11", Tags: ParseTags("host=hostB")}},
+		{&influxql.StringPoint{Time: 5, Value: "21", Tags: ParseTags("host=hostA")}},
+		{&influxql.StringPoint{Time: 20, Value: "8", Tags: ParseTags("host=hostB")}},
+	}) {
+		t.Fatalf("unexpected points: %s", spew.Sdump(a))
+	}
+}
+
+// Ensure that a boolean iterator can be created for a modBooleanl.
+func TestCallIterator_Mode_Boolean(t *testing.T) {
+	itr, _ := influxql.NewModeIterator(&BooleanIterator{Points: []influxql.BooleanPoint{
+		{Time: 0, Value: true, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 1, Value: false, Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 1, Value: true, Tags: ParseTags("region=us-west,host=hostA")},
+		{Time: 2, Value: true, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 3, Value: true, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 4, Value: false, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 6, Value: false, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 7, Value: false, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 8, Value: false, Tags: ParseTags("region=us-east,host=hostA")},
+		{Time: 22, Value: false, Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 23, Value: true, Tags: ParseTags("region=us-west,host=hostB")},
+		{Time: 24, Value: true, Tags: ParseTags("region=us-west,host=hostB")},
+	}},
+		influxql.IteratorOptions{
+			Expr:       MustParseExpr(`mode("value")`),
+			Dimensions: []string{"host"},
+			Interval:   influxql.Interval{Duration: 5 * time.Nanosecond},
+		},
+	)
+
+	if a, err := Iterators([]influxql.Iterator{itr}).ReadAll(); err != nil {
+		t.Fatalf("unexpected error: %s", err)
+	} else if !deep.Equal(a, [][]influxql.Point{
+		{&influxql.BooleanPoint{Time: 0, Value: true, Tags: ParseTags("host=hostA")}},
+		{&influxql.BooleanPoint{Time: 1, Value: false, Tags: ParseTags("host=hostB")}},
+		{&influxql.BooleanPoint{Time: 5, Value: false, Tags: ParseTags("host=hostA")}},
+		{&influxql.BooleanPoint{Time: 20, Value: true, Tags: ParseTags("host=hostB")}},
+	}) {
+		t.Fatalf("unexpected points: %s", spew.Sdump(a))
+	}
+}
+
 func TestNewCallIterator_UnsupportedExprName(t *testing.T) {
 	_, err := influxql.NewCallIterator(
 		&FloatIterator{},
@@ -731,6 +871,33 @@ func benchmarkDistinctIterator(b *testing.B, pointN int) {
 	}
 }
 
+func BenchmarkModeIterator_1K(b *testing.B)   { benchmarkModeIterator(b, 1000) }
+func BenchmarkModeIterator_100K(b *testing.B) { benchmarkModeIterator(b, 100000) }
+func BenchmarkModeIterator_1M(b *testing.B)   { benchmarkModeIterator(b, 1000000) }
+
+func benchmarkModeIterator(b *testing.B, pointN int) {
+	b.ReportAllocs()
+
+	for i := 0; i < b.N; i++ {
+		// Create a lightweight point generator.
+		p := influxql.FloatPoint{Name: "cpu"}
+		input := FloatPointGenerator{
+			N: pointN,
+			Fn: func(i int) *influxql.FloatPoint {
+				p.Value = float64(10)
+				return &p
+			},
+		}
+
+		// Execute call against input.
+		itr, err := influxql.NewModeIterator(&input, influxql.IteratorOptions{})
+		if err != nil {
+			b.Fatal(err)
+		}
+		influxql.DrainIterator(itr)
+	}
+}
+
 type FloatPointGenerator struct {
 	i  int
 	N  int
@@ -740,11 +907,11 @@ type FloatPointGenerator struct {
 func (g *FloatPointGenerator) Close() error                  { return nil }
 func (g *FloatPointGenerator) Stats() influxql.IteratorStats { return influxql.IteratorStats{} }
 
-func (g *FloatPointGenerator) Next() *influxql.FloatPoint {
+func (g *FloatPointGenerator) Next() (*influxql.FloatPoint, error) {
 	if g.i == g.N {
-		return nil
+		return nil, nil
 	}
 	p := g.Fn(g.i)
 	g.i++
-	return p
+	return p, nil
 }
