@@ -2,16 +2,14 @@ package udp // import "github.com/influxdata/influxdb/services/udp"
 
 import (
 	"errors"
-	"expvar"
 	"io"
 	"log"
 	"net"
 	"os"
-	"strings"
 	"sync"
+	"sync/atomic"
 	"time"
 
-	"github.com/influxdata/influxdb"
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/services/meta"
 	"github.com/influxdata/influxdb/tsdb"
@@ -30,7 +28,7 @@ const (
 	statBytesReceived       = "bytesRx"
 	statPointsParseFail     = "pointsParseFail"
 	statReadFail            = "readFail"
-	statBatchesTrasmitted   = "batchesTx"
+	statBatchesTransmitted  = "batchesTx"
 	statPointsTransmitted   = "pointsTx"
 	statBatchesTransmitFail = "batchesTxFail"
 )
@@ -58,8 +56,9 @@ type Service struct {
 		CreateDatabase(name string) (*meta.DatabaseInfo, error)
 	}
 
-	Logger  *log.Logger
-	statMap *expvar.Map
+	Logger   *log.Logger
+	stats    *Statistics
+	statTags models.Tags
 }
 
 // NewService returns a new instance of Service.
@@ -71,17 +70,13 @@ func NewService(c Config) *Service {
 		parserChan: make(chan []byte, parserChanLen),
 		batcher:    tsdb.NewPointBatcher(d.BatchSize, d.BatchPending, time.Duration(d.BatchTimeout)),
 		Logger:     log.New(os.Stderr, "[udp] ", log.LstdFlags),
+		stats:      &Statistics{},
+		statTags:   map[string]string{"bind": d.BindAddress},
 	}
 }
 
 // Open starts the service
 func (s *Service) Open() (err error) {
-	// Configure expvar monitoring. It's OK to do this even if the service fails to open and
-	// should be done before any data could arrive for the service.
-	key := strings.Join([]string{"udp", s.config.BindAddress}, ":")
-	tags := map[string]string{"bind": s.config.BindAddress}
-	s.statMap = influxdb.NewStatistics(key, "udp", tags)
-
 	if s.config.BindAddress == "" {
 		return errors.New("bind address has to be specified in config")
 	}
@@ -124,6 +119,34 @@ func (s *Service) Open() (err error) {
 	return nil
 }
 
+// Statistics maintains statistics for the UDP service.
+type Statistics struct {
+	PointsReceived      int64
+	BytesReceived       int64
+	PointsParseFail     int64
+	ReadFail            int64
+	BatchesTransmitted  int64
+	PointsTransmitted   int64
+	BatchesTransmitFail int64
+}
+
+// Statistics returns statistics for periodic monitoring.
+func (s *Service) Statistics(tags map[string]string) []models.Statistic {
+	return []models.Statistic{{
+		Name: "udp",
+		Tags: s.statTags,
+		Values: map[string]interface{}{
+			statPointsReceived:      atomic.LoadInt64(&s.stats.PointsReceived),
+			statBytesReceived:       atomic.LoadInt64(&s.stats.BytesReceived),
+			statPointsParseFail:     atomic.LoadInt64(&s.stats.PointsParseFail),
+			statReadFail:            atomic.LoadInt64(&s.stats.ReadFail),
+			statBatchesTransmitted:  atomic.LoadInt64(&s.stats.BatchesTransmitted),
+			statPointsTransmitted:   atomic.LoadInt64(&s.stats.PointsTransmitted),
+			statBatchesTransmitFail: atomic.LoadInt64(&s.stats.BatchesTransmitFail),
+		},
+	}}
+}
+
 func (s *Service) writer() {
 	defer s.wg.Done()
 
@@ -131,11 +154,11 @@ func (s *Service) writer() {
 		select {
 		case batch := <-s.batcher.Out():
 			if err := s.PointsWriter.WritePoints(s.config.Database, s.config.RetentionPolicy, models.ConsistencyLevelAny, batch); err == nil {
-				s.statMap.Add(statBatchesTrasmitted, 1)
-				s.statMap.Add(statPointsTransmitted, int64(len(batch)))
+				atomic.AddInt64(&s.stats.BatchesTransmitted, 1)
+				atomic.AddInt64(&s.stats.PointsTransmitted, int64(len(batch)))
 			} else {
 				s.Logger.Printf("failed to write point batch to database %q: %s", s.config.Database, err)
-				s.statMap.Add(statBatchesTransmitFail, 1)
+				atomic.AddInt64(&s.stats.BatchesTransmitFail, 1)
 			}
 
 		case <-s.done:
@@ -159,11 +182,11 @@ func (s *Service) serve() {
 			// Keep processing.
 			n, _, err := s.conn.ReadFromUDP(buf)
 			if err != nil {
-				s.statMap.Add(statReadFail, 1)
+				atomic.AddInt64(&s.stats.ReadFail, 1)
 				s.Logger.Printf("Failed to read UDP message: %s", err)
 				continue
 			}
-			s.statMap.Add(statBytesReceived, int64(n))
+			atomic.AddInt64(&s.stats.BytesReceived, int64(n))
 
 			bufCopy := make([]byte, n)
 			copy(bufCopy, buf[:n])
@@ -182,7 +205,7 @@ func (s *Service) parser() {
 		case buf := <-s.parserChan:
 			points, err := models.ParsePointsWithPrecision(buf, time.Now().UTC(), s.config.Precision)
 			if err != nil {
-				s.statMap.Add(statPointsParseFail, 1)
+				atomic.AddInt64(&s.stats.PointsParseFail, 1)
 				s.Logger.Printf("Failed to parse points: %s", err)
 				continue
 			}
@@ -190,7 +213,7 @@ func (s *Service) parser() {
 			for _, point := range points {
 				s.batcher.In() <- point
 			}
-			s.statMap.Add(statPointsReceived, int64(len(points)))
+			atomic.AddInt64(&s.stats.PointsReceived, int64(len(points)))
 		}
 	}
 }
