@@ -450,7 +450,7 @@ func TestEngine_CreateIterator_Aux(t *testing.T) {
 
 	itr, err := e.CreateIterator(influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr(`value`),
-		Aux:        []string{"F"},
+		Aux:        []influxql.VarRef{{Val: "F"}},
 		Dimensions: []string{"host"},
 		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		StartTime:  influxql.MinTime,
@@ -543,6 +543,60 @@ func TestEngine_CreateIterator_Condition(t *testing.T) {
 	}
 }
 
+// Ensures that deleting series from TSM files with multiple fields removes all the
+/// series
+func TestEngine_DeleteSeries(t *testing.T) {
+	// Generate temporary file.
+	f, _ := ioutil.TempFile("", "tsm")
+	f.Close()
+	os.Remove(f.Name())
+	walPath := filepath.Join(f.Name(), "wal")
+	os.MkdirAll(walPath, 0777)
+	defer os.RemoveAll(f.Name())
+
+	// Create a few points.
+	p1 := MustParsePointString("cpu,host=A value=1.1 1000000000")
+	p2 := MustParsePointString("cpu,host=B value=1.2 2000000000")
+	p3 := MustParsePointString("cpu,host=A sum=1.3 3000000000")
+
+	// Write those points to the engine.
+	e := tsm1.NewEngine(f.Name(), walPath, tsdb.NewEngineOptions()).(*tsm1.Engine)
+
+	// mock the planner so compactions don't run during the test
+	e.CompactionPlan = &mockPlanner{}
+
+	if err := e.Open(); err != nil {
+		t.Fatalf("failed to open tsm1 engine: %s", err.Error())
+	}
+
+	if err := e.WritePoints([]models.Point{p1, p2, p3}); err != nil {
+		t.Fatalf("failed to write points: %s", err.Error())
+	}
+	if err := e.WriteSnapshot(); err != nil {
+		t.Fatalf("failed to snapshot: %s", err.Error())
+	}
+
+	keys := e.FileStore.Keys()
+	if exp, got := 3, len(keys); exp != got {
+		t.Fatalf("series count mismatch: exp %v, got %v", exp, got)
+	}
+
+	if err := e.DeleteSeries([]string{"cpu,host=A"}); err != nil {
+		t.Fatalf("failed to delete series: %v", err)
+	}
+
+	keys = e.FileStore.Keys()
+	if exp, got := 1, len(keys); exp != got {
+		t.Fatalf("series count mismatch: exp %v, got %v", exp, got)
+	}
+
+	exp := "cpu,host=B#!~#value"
+	if _, ok := keys[exp]; !ok {
+		t.Fatalf("wrong series deleted: exp %v, got %v", exp, keys)
+	}
+
+}
+
 func BenchmarkEngine_CreateIterator_Count_1K(b *testing.B) {
 	benchmarkEngineCreateIteratorCount(b, 1000)
 }
@@ -573,6 +627,39 @@ func BenchmarkEngine_CreateIterator_Limit_1M(b *testing.B) {
 	benchmarkEngineCreateIteratorLimit(b, 1000000)
 }
 
+func BenchmarkEngine_WritePoints_10(b *testing.B) {
+	benchmarkEngine_WritePoints(b, 10)
+}
+func BenchmarkEngine_WritePoints_100(b *testing.B) {
+	benchmarkEngine_WritePoints(b, 100)
+}
+func BenchmarkEngine_WritePoints_1000(b *testing.B) {
+	benchmarkEngine_WritePoints(b, 1000)
+}
+
+func benchmarkEngine_WritePoints(b *testing.B, batchSize int) {
+	e := MustOpenEngine()
+	defer e.Close()
+
+	e.Index().CreateMeasurementIndexIfNotExists("cpu")
+	e.MeasurementFields("cpu").CreateFieldIfNotExists("value", influxql.Float, false)
+
+	p := MustParsePointString("cpu value=1.2")
+	pp := make([]models.Point, 0, batchSize)
+	for i := 0; i < batchSize; i++ {
+		pp = append(pp, p)
+	}
+
+	b.ResetTimer()
+	b.ReportAllocs()
+	for i := 0; i < b.N; i++ {
+		err := e.WritePoints(pp)
+		if err != nil {
+			b.Fatal(err)
+		}
+	}
+}
+
 func benchmarkEngineCreateIteratorLimit(b *testing.B, pointN int) {
 	benchmarkIterator(b, influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr("value"),
@@ -750,6 +837,7 @@ type mockPlanner struct{}
 
 func (m *mockPlanner) Plan(lastWrite time.Time) []tsm1.CompactionGroup { return nil }
 func (m *mockPlanner) PlanLevel(level int) []tsm1.CompactionGroup      { return nil }
+func (m *mockPlanner) PlanOptimize() []tsm1.CompactionGroup            { return nil }
 
 // ParseTags returns an instance of Tags for a comma-delimited list of key/values.
 func ParseTags(s string) influxql.Tags {
