@@ -354,6 +354,94 @@ func Test_IntegerEncoder_CounterRLE(t *testing.T) {
 		t.Fatalf("unexpected error: %v", err)
 	}
 
+	if b[0]>>4 != intCompressedRLE {
+		t.Fatalf("unexpected encoding format: expected RLE, got %v", b[0]>>4)
+	}
+
+	// Should use 1 header byte, 8 byte first value, 1 var-byte for delta and 1 var-byte for
+	// count of deltas in this particular RLE.
+	if exp := 11; len(b) != exp {
+		t.Fatalf("encoded length mismatch: got %v, exp %v", len(b), exp)
+	}
+
+	var dec IntegerDecoder
+	dec.SetBytes(b)
+	i := 0
+	for dec.Next() {
+		if i > len(values) {
+			t.Fatalf("read too many values: got %v, exp %v", i, len(values))
+		}
+
+		if values[i] != dec.Read() {
+			t.Fatalf("read value %d mismatch: got %v, exp %v", i, dec.Read(), values[i])
+		}
+		i += 1
+	}
+
+	if i != len(values) {
+		t.Fatalf("failed to read enough values: got %v, exp %v", i, len(values))
+	}
+}
+
+func Test_IntegerEncoder_Descending(t *testing.T) {
+	enc := NewIntegerEncoder()
+	values := []int64{
+		7094, 4472, 1850,
+	}
+
+	for _, v := range values {
+		enc.Write(v)
+	}
+
+	b, err := enc.Bytes()
+	if err != nil {
+		t.Fatalf("unexpected error: %v", err)
+	}
+
+	if b[0]>>4 != intCompressedRLE {
+		t.Fatalf("unexpected encoding format: expected simple, got %v", b[0]>>4)
+	}
+
+	// Should use 1 header byte, 8 byte first value, 1 var-byte for delta and 1 var-byte for
+	// count of deltas in this particular RLE.
+	if exp := 12; len(b) != exp {
+		t.Fatalf("encoded length mismatch: got %v, exp %v", len(b), exp)
+	}
+
+	var dec IntegerDecoder
+	dec.SetBytes(b)
+	i := 0
+	for dec.Next() {
+		if i > len(values) {
+			t.Fatalf("read too many values: got %v, exp %v", i, len(values))
+		}
+
+		if values[i] != dec.Read() {
+			t.Fatalf("read value %d mismatch: got %v, exp %v", i, dec.Read(), values[i])
+		}
+		i += 1
+	}
+
+	if i != len(values) {
+		t.Fatalf("failed to read enough values: got %v, exp %v", i, len(values))
+	}
+}
+
+func Test_IntegerEncoder_Flat(t *testing.T) {
+	enc := NewIntegerEncoder()
+	values := []int64{
+		1, 1, 1, 1,
+	}
+
+	for _, v := range values {
+		enc.Write(v)
+	}
+
+	b, err := enc.Bytes()
+	if err != nil {
+		t.Fatalf("unexpected error: %v", err)
+	}
+
 	if b[0]>>4 != intCompressedRLE {
 		t.Fatalf("unexpected encoding format: expected simple, got %v", b[0]>>4)
 	}
@@ -464,6 +552,25 @@ func Test_IntegerEncoder_Quick(t *testing.T) {
 	}, nil)
 }
 
+func Test_IntegerDecoder_Corrupt(t *testing.T) {
+	cases := []string{
+		"",                     // Empty
+		"\x00abc",              // Uncompressed: less than 8 bytes
+		"\x10abc",              // Packed: less than 8 bytes
+		"\x20abc",              // RLE: less than 8 bytes
+		"\x2012345678\x90",     // RLE: valid starting value but invalid delta value
+		"\x2012345678\x01\x90", // RLE: valid starting, valid delta value, invalid repeat value
+	}
+
+	for _, c := range cases {
+		var dec IntegerDecoder
+		dec.SetBytes([]byte(c))
+		if dec.Next() {
+			t.Fatalf("exp next == false, got true")
+		}
+	}
+}
+
 func BenchmarkIntegerEncoderRLE(b *testing.B) {
 	enc := NewIntegerEncoder()
 	x := make([]int64, 1024)
