@@ -1,34 +1,37 @@
 package tsdb
 
 import (
-	"encoding/binary"
-	"encoding/json"
 	"errors"
-	"expvar"
 	"fmt"
 	"io"
 	"log"
-	"math"
 	"os"
+	"path/filepath"
 	"sort"
-	"strings"
 	"sync"
+	"sync/atomic"
 	"time"
 
 	"github.com/gogo/protobuf/proto"
-	"github.com/influxdata/influxdb"
 	"github.com/influxdata/influxdb/influxql"
 	"github.com/influxdata/influxdb/models"
 	internal "github.com/influxdata/influxdb/tsdb/internal"
 )
 
+// monitorStatInterval is the interval at which the shard is inspected
+// for the purpose of determining certain monitoring statistics.
+const monitorStatInterval = 30 * time.Second
+
 const (
-	statWriteReq        = "writeReq"
-	statSeriesCreate    = "seriesCreate"
-	statFieldsCreate    = "fieldsCreate"
-	statWritePointsFail = "writePointsFail"
-	statWritePointsOK   = "writePointsOk"
-	statWriteBytes      = "writeBytes"
+	statWriteReq       = "writeReq"
+	statWriteReqOK     = "writeReqOk"
+	statWriteReqErr    = "writeReqErr"
+	statSeriesCreate   = "seriesCreate"
+	statFieldsCreate   = "fieldsCreate"
+	statWritePointsErr = "writePointsErr"
+	statWritePointsOK  = "writePointsOk"
+	statWriteBytes     = "writeBytes"
+	statDiskBytes      = "diskBytes"
 )
 
 var (
@@ -48,6 +51,10 @@ var (
 	// ErrEngineClosed is returned when a caller attempts indirectly to
 	// access the shard's underlying engine.
 	ErrEngineClosed = errors.New("engine is closed")
+
+	// ErrShardDisabled is returned when a the shard is not available for
+	// queries or writes.
+	ErrShardDisabled = errors.New("shard is disabled")
 )
 
 // A ShardError implements the error interface, and contains extra
@@ -84,57 +91,115 @@ type Shard struct {
 
 	options EngineOptions
 
-	mu     sync.RWMutex
-	engine Engine
+	mu      sync.RWMutex
+	engine  Engine
+	closing chan struct{}
+	enabled bool
 
 	// expvar-based stats.
-	statMap *expvar.Map
+	stats    *ShardStatistics
+	statTags models.Tags
 
 	logger *log.Logger
+	// used by logger. Referenced so it can be passed down to new caches.
+	logOutput io.Writer
 
-	// The writer used by the logger.
-	LogOutput io.Writer
+	EnableOnOpen bool
 }
 
 // NewShard returns a new initialized Shard. walPath doesn't apply to the b1 type index
 func NewShard(id uint64, index *DatabaseIndex, path string, walPath string, options EngineOptions) *Shard {
-	// Configure statistics collection.
-	key := fmt.Sprintf("shard:%s:%d", path, id)
 	db, rp := DecodeStorePath(path)
-	tags := map[string]string{
-		"path":            path,
-		"id":              fmt.Sprintf("%d", id),
-		"engine":          options.EngineVersion,
-		"database":        db,
-		"retentionPolicy": rp,
-	}
-	statMap := influxdb.NewStatistics(key, "shard", tags)
-
 	s := &Shard{
 		index:   index,
 		id:      id,
 		path:    path,
 		walPath: walPath,
 		options: options,
+		closing: make(chan struct{}),
+
+		stats: &ShardStatistics{},
+		statTags: map[string]string{
+			"path":            path,
+			"id":              fmt.Sprintf("%d", id),
+			"database":        db,
+			"retentionPolicy": rp,
+		},
 
 		database:        db,
 		retentionPolicy: rp,
 
-		statMap:   statMap,
-		LogOutput: os.Stderr,
+		logger:       log.New(os.Stderr, "[shard] ", log.LstdFlags),
+		logOutput:    os.Stderr,
+		EnableOnOpen: true,
 	}
-	s.SetLogOutput(os.Stderr)
 	return s
 }
 
-// SetLogOutput sets the writer to which log output will be written. It must
-// not be called after the Open method has been called.
+// SetLogOutput sets the writer to which log output will be written. It is safe
+// for concurrent use.
 func (s *Shard) SetLogOutput(w io.Writer) {
-	s.LogOutput = w
-	s.logger = log.New(w, "[shard] ", log.LstdFlags)
-	if !s.closed() {
+	s.logger.SetOutput(w)
+	if err := s.ready(); err == nil {
 		s.engine.SetLogOutput(w)
 	}
+
+	s.mu.Lock()
+	s.logOutput = w
+	s.mu.Unlock()
+}
+
+// SetEnabled enables the shard for queries and write.  When disabled, all
+// writes and queries return an error and compactions are stopped for the shard.
+func (s *Shard) SetEnabled(enabled bool) {
+	s.mu.Lock()
+	// Prevent writes and queries
+	s.enabled = enabled
+	if s.engine != nil {
+		// Disable background compactions and snapshotting
+		s.engine.SetEnabled(enabled)
+	}
+	s.mu.Unlock()
+}
+
+// ShardStatistics maintains statistics for a shard.
+type ShardStatistics struct {
+	WriteReq       int64
+	WriteReqOK     int64
+	WriteReqErr    int64
+	SeriesCreated  int64
+	FieldsCreated  int64
+	WritePointsErr int64
+	WritePointsOK  int64
+	BytesWritten   int64
+	DiskBytes      int64
+}
+
+// Statistics returns statistics for periodic monitoring.
+func (s *Shard) Statistics(tags map[string]string) []models.Statistic {
+	if err := s.ready(); err != nil {
+		return nil
+	}
+
+	tags = s.statTags.Merge(tags)
+	seriesN, _ := s.engine.SeriesCount()
+	statistics := []models.Statistic{{
+		Name: "shard",
+		Tags: models.Tags(tags).Merge(map[string]string{"engine": s.options.EngineVersion}),
+		Values: map[string]interface{}{
+			statWriteReq:       atomic.LoadInt64(&s.stats.WriteReq),
+			statWriteReqOK:     atomic.LoadInt64(&s.stats.WriteReqOK),
+			statWriteReqErr:    atomic.LoadInt64(&s.stats.WriteReqErr),
+			statSeriesCreate:   seriesN,
+			statFieldsCreate:   atomic.LoadInt64(&s.stats.FieldsCreated),
+			statWritePointsErr: atomic.LoadInt64(&s.stats.WritePointsErr),
+			statWritePointsOK:  atomic.LoadInt64(&s.stats.WritePointsOK),
+			statWriteBytes:     atomic.LoadInt64(&s.stats.BytesWritten),
+			statDiskBytes:      atomic.LoadInt64(&s.stats.DiskBytes),
+		},
+	}}
+	statistics = append(statistics, s.engine.Statistics(tags)...)
+	return statistics
 }
 
 // Path returns the path set on the shard when it was created.
@@ -156,32 +221,53 @@ func (s *Shard) Open() error {
 		if err != nil {
 			return err
 		}
-		s.engine = e
 
 		// Set log output on the engine.
-		s.engine.SetLogOutput(s.LogOutput)
+		e.SetLogOutput(s.logOutput)
+
+		// Disable compactions while loading the index
+		e.SetEnabled(false)
 
 		// Open engine.
-		if err := s.engine.Open(); err != nil {
+		if err := e.Open(); err != nil {
 			return err
 		}
 
 		// Load metadata index.
 		start := time.Now()
-		if err := s.engine.LoadMetadataIndex(s.id, s.index); err != nil {
+		if err := e.LoadMetadataIndex(s.id, s.index); err != nil {
 			return err
 		}
+
+		count := s.index.SeriesShardN(s.id)
+		atomic.AddInt64(&s.stats.SeriesCreated, int64(count))
+
+		s.engine = e
+
 		s.logger.Printf("%s database index loaded in %s", s.path, time.Now().Sub(start))
 
+		go s.monitorSize()
+
 		return nil
 	}(); err != nil {
 		s.close()
 		return NewShardError(s.id, err)
 	}
 
+	if s.EnableOnOpen {
+		// enable writes, queries and compactions
+		s.SetEnabled(true)
+	}
+
 	return nil
 }
 
+// UnloadIndex removes all references to this shard from the DatabaseIndex
+func (s *Shard) UnloadIndex() {
+	// Don't leak our shard ID and series keys in the index
+	s.index.RemoveShard(s.id)
+}
+
 // Close shuts down the shard's store.
 func (s *Shard) Close() error {
 	s.mu.Lock()
@@ -194,8 +280,15 @@ func (s *Shard) close() error {
 		return nil
 	}
 
+	// Close the closing channel at most once.
+	select {
+	case <-s.closing:
+	default:
+		close(s.closing)
+	}
+
 	// Don't leak our shard ID and series keys in the index
-	s.index.RemoveShard(s.id)
+	s.UnloadIndex()
 
 	err := s.engine.Close()
 	if err == nil {
@@ -204,32 +297,50 @@ func (s *Shard) close() error {
 	return err
 }
 
-// closed determines if the Shard is closed.
-func (s *Shard) closed() bool {
+// ready determines if the Shard is ready for queries or writes.
+// It returns nil if ready, otherwise ErrShardClosed or ErrShardDiabled
+func (s *Shard) ready() error {
+	var err error
+
 	s.mu.RLock()
-	closed := s.engine == nil
+	if s.engine == nil {
+		err = ErrEngineClosed
+	} else if !s.enabled {
+		err = ErrShardDisabled
+	}
 	s.mu.RUnlock()
-	return closed
+	return err
 }
 
 // DiskSize returns the size on disk of this shard
 func (s *Shard) DiskSize() (int64, error) {
-	stats, err := os.Stat(s.path)
+	var size int64
+	err := filepath.Walk(s.path, func(_ string, fi os.FileInfo, err error) error {
+		if err != nil {
+			return err
+		}
+
+		if !fi.IsDir() {
+			size += fi.Size()
+		}
+		return err
+	})
 	if err != nil {
 		return 0, err
 	}
-	return stats.Size(), nil
-}
 
-// FieldCodec returns the field encoding for a measurement.
-// TODO: this is temporarily exported to make tx.go work. When the query engine gets refactored
-// into the tsdb package this should be removed. No one outside tsdb should know the underlying field encoding scheme.
-func (s *Shard) FieldCodec(measurementName string) *FieldCodec {
-	m := s.engine.MeasurementFields(measurementName)
-	if m == nil {
-		return NewFieldCodec(nil)
-	}
-	return m.Codec
+	err = filepath.Walk(s.walPath, func(_ string, fi os.FileInfo, err error) error {
+		if err != nil {
+			return err
+		}
+
+		if !fi.IsDir() {
+			size += fi.Size()
+		}
+		return err
+	})
+
+	return size, err
 }
 
 // FieldCreate holds information for a field to create on a measurement
@@ -246,20 +357,20 @@ type SeriesCreate struct {
 
 // WritePoints will write the raw data points and any new metadata to the index in the shard
 func (s *Shard) WritePoints(points []models.Point) error {
-	if s.closed() {
-		return ErrEngineClosed
+	if err := s.ready(); err != nil {
+		return err
 	}
 
 	s.mu.RLock()
 	defer s.mu.RUnlock()
 
-	s.statMap.Add(statWriteReq, 1)
+	atomic.AddInt64(&s.stats.WriteReq, 1)
 
 	fieldsToCreate, err := s.validateSeriesAndFields(points)
 	if err != nil {
 		return err
 	}
-	s.statMap.Add(statFieldsCreate, int64(len(fieldsToCreate)))
+	atomic.AddInt64(&s.stats.FieldsCreated, int64(len(fieldsToCreate)))
 
 	// add any new fields and keep track of what needs to be saved
 	if err := s.createFieldsAndMeasurements(fieldsToCreate); err != nil {
@@ -268,17 +379,19 @@ func (s *Shard) WritePoints(points []models.Point) error {
 
 	// Write to the engine.
 	if err := s.engine.WritePoints(points); err != nil {
-		s.statMap.Add(statWritePointsFail, 1)
+		atomic.AddInt64(&s.stats.WritePointsErr, int64(len(points)))
+		atomic.AddInt64(&s.stats.WriteReqErr, 1)
 		return fmt.Errorf("engine: %s", err)
 	}
-	s.statMap.Add(statWritePointsOK, int64(len(points)))
+	atomic.AddInt64(&s.stats.WritePointsOK, int64(len(points)))
+	atomic.AddInt64(&s.stats.WriteReqOK, 1)
 
 	return nil
 }
 
 func (s *Shard) ContainsSeries(seriesKeys []string) (map[string]bool, error) {
-	if s.closed() {
-		return nil, ErrEngineClosed
+	if err := s.ready(); err != nil {
+		return nil, err
 	}
 
 	return s.engine.ContainsSeries(seriesKeys)
@@ -286,8 +399,8 @@ func (s *Shard) ContainsSeries(seriesKeys []string) (map[string]bool, error) {
 
 // DeleteSeries deletes a list of series.
 func (s *Shard) DeleteSeries(seriesKeys []string) error {
-	if s.closed() {
-		return ErrEngineClosed
+	if err := s.ready(); err != nil {
+		return err
 	}
 	if err := s.engine.DeleteSeries(seriesKeys); err != nil {
 		return err
@@ -297,9 +410,10 @@ func (s *Shard) DeleteSeries(seriesKeys []string) error {
 
 // DeleteSeriesRange deletes all values from for seriesKeys between min and max (inclusive)
 func (s *Shard) DeleteSeriesRange(seriesKeys []string, min, max int64) error {
-	if s.closed() {
-		return ErrEngineClosed
+	if err := s.ready(); err != nil {
+		return err
 	}
+
 	if err := s.engine.DeleteSeriesRange(seriesKeys, min, max); err != nil {
 		return err
 	}
@@ -309,8 +423,8 @@ func (s *Shard) DeleteSeriesRange(seriesKeys []string, min, max int64) error {
 
 // DeleteMeasurement deletes a measurement and all underlying series.
 func (s *Shard) DeleteMeasurement(name string, seriesKeys []string) error {
-	if s.closed() {
-		return ErrEngineClosed
+	if err := s.ready(); err != nil {
+		return err
 	}
 
 	if err := s.engine.DeleteMeasurement(name, seriesKeys); err != nil {
@@ -352,8 +466,12 @@ func (s *Shard) validateSeriesAndFields(points []models.Point) ([]*FieldCreate,
 		key := string(p.Key())
 		ss := s.index.Series(key)
 		if ss == nil {
+			if s.options.Config.MaxSeriesPerDatabase > 0 && len(s.index.series)+1 > s.options.Config.MaxSeriesPerDatabase {
+				return nil, fmt.Errorf("max series per database exceeded: %s", key)
+			}
+
 			ss = NewSeries(key, p.Tags())
-			s.statMap.Add(statSeriesCreate, 1)
+			atomic.AddInt64(&s.stats.SeriesCreated, 1)
 		}
 
 		ss = s.index.CreateSeriesIndexIfNotExists(p.Name(), ss)
@@ -389,31 +507,32 @@ func (s *Shard) validateSeriesAndFields(points []models.Point) ([]*FieldCreate,
 
 // SeriesCount returns the number of series buckets on the shard.
 func (s *Shard) SeriesCount() (int, error) {
-	if s.closed() {
-		return 0, ErrEngineClosed
+	if err := s.ready(); err != nil {
+		return 0, err
 	}
 	return s.engine.SeriesCount()
 }
 
 // WriteTo writes the shard's data to w.
 func (s *Shard) WriteTo(w io.Writer) (int64, error) {
-	if s.closed() {
-		return 0, ErrEngineClosed
+	if err := s.ready(); err != nil {
+		return 0, err
 	}
 	n, err := s.engine.WriteTo(w)
-	s.statMap.Add(statWriteBytes, int64(n))
+	atomic.AddInt64(&s.stats.BytesWritten, int64(n))
 	return n, err
 }
 
 // CreateIterator returns an iterator for the data in the shard.
 func (s *Shard) CreateIterator(opt influxql.IteratorOptions) (influxql.Iterator, error) {
-	if s.closed() {
-		return nil, ErrEngineClosed
+	if err := s.ready(); err != nil {
+		return nil, err
 	}
 
 	if influxql.Sources(opt.Sources).HasSystemSource() {
 		return s.createSystemIterator(opt)
 	}
+	opt.Sources = influxql.Sources(opt.Sources).Filter(s.database, s.retentionPolicy)
 	return s.engine.CreateIterator(opt)
 }
 
@@ -428,22 +547,49 @@ func (s *Shard) createSystemIterator(opt influxql.IteratorOptions) (influxql.Ite
 	switch m.Name {
 	case "_fieldKeys":
 		return NewFieldKeysIterator(s, opt)
-	case "_measurements":
-		return NewMeasurementIterator(s, opt)
 	case "_series":
 		return NewSeriesIterator(s, opt)
 	case "_tagKeys":
 		return NewTagKeysIterator(s, opt)
-	case "_tags":
-		return NewTagValuesIterator(s, opt)
 	default:
 		return nil, fmt.Errorf("unknown system source: %s", m.Name)
 	}
 }
 
 // FieldDimensions returns unique sets of fields and dimensions across a list of sources.
-func (s *Shard) FieldDimensions(sources influxql.Sources) (fields, dimensions map[string]struct{}, err error) {
-	fields = make(map[string]struct{})
+func (s *Shard) FieldDimensions(sources influxql.Sources) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error) {
+	if err := s.ready(); err != nil {
+		return nil, nil, err
+	}
+
+	if sources.HasSystemSource() {
+		// Only support a single system source.
+		if len(sources) > 1 {
+			return nil, nil, errors.New("cannot select from multiple system sources")
+		}
+
+		switch m := sources[0].(type) {
+		case *influxql.Measurement:
+			switch m.Name {
+			case "_fieldKeys":
+				return map[string]influxql.DataType{
+					"fieldKey":  influxql.String,
+					"fieldType": influxql.String,
+				}, nil, nil
+			case "_series":
+				return map[string]influxql.DataType{
+					"key": influxql.String,
+				}, nil, nil
+			case "_tagKeys":
+				return map[string]influxql.DataType{
+					"tagKey": influxql.String,
+				}, nil, nil
+			}
+		}
+		return nil, nil, nil
+	}
+
+	fields = make(map[string]influxql.DataType)
 	dimensions = make(map[string]struct{})
 
 	for _, src := range sources {
@@ -456,8 +602,11 @@ func (s *Shard) FieldDimensions(sources influxql.Sources) (fields, dimensions ma
 			}
 
 			// Append fields and dimensions.
-			for _, name := range mm.FieldNames() {
-				fields[name] = struct{}{}
+			mf := s.engine.MeasurementFields(m.Name)
+			if mf != nil {
+				for name, typ := range mf.FieldSet() {
+					fields[name] = typ
+				}
 			}
 			for _, key := range mm.TagKeys() {
 				dimensions[key] = struct{}{}
@@ -468,30 +617,6 @@ func (s *Shard) FieldDimensions(sources influxql.Sources) (fields, dimensions ma
 	return
 }
 
-// SeriesKeys returns a list of series in the shard.
-func (s *Shard) SeriesKeys(opt influxql.IteratorOptions) (influxql.SeriesList, error) {
-	if s.closed() {
-		return nil, ErrEngineClosed
-	}
-
-	if influxql.Sources(opt.Sources).HasSystemSource() {
-		// Only support a single system source.
-		if len(opt.Sources) > 1 {
-			return nil, errors.New("cannot select from multiple system sources")
-		}
-
-		// Meta queries don't need to know the series name and
-		// always have a single series of strings.
-		auxFields := make([]influxql.DataType, len(opt.Aux))
-		for i := range auxFields {
-			auxFields[i] = influxql.String
-		}
-		return []influxql.Series{{Aux: auxFields}}, nil
-	}
-
-	return s.engine.SeriesKeys(opt)
-}
-
 // ExpandSources expands regex sources and removes duplicates.
 // NOTE: sources must be normalized (db and rp set) before calling this function.
 func (s *Shard) ExpandSources(sources influxql.Sources) (influxql.Sources, error) {
@@ -539,6 +664,54 @@ func (s *Shard) ExpandSources(sources influxql.Sources) (influxql.Sources, error
 	return expanded, nil
 }
 
+// Restore restores data to the underlying engine for the shard.
+// The shard is reopened after restore.
+func (s *Shard) Restore(r io.Reader, basePath string) error {
+	s.mu.Lock()
+
+	// Restore to engine.
+	if err := s.engine.Restore(r, basePath); err != nil {
+		s.mu.Unlock()
+		return err
+	}
+
+	s.mu.Unlock()
+
+	// Close shard.
+	if err := s.Close(); err != nil {
+		return err
+	}
+
+	// Reopen engine.
+	return s.Open()
+}
+
+// CreateSnapshot will return a path to a temp directory
+// containing hard links to the underlying shard files
+func (s *Shard) CreateSnapshot() (string, error) {
+	s.mu.RLock()
+	defer s.mu.RUnlock()
+	return s.engine.CreateSnapshot()
+}
+
+func (s *Shard) monitorSize() {
+	t := time.NewTicker(monitorStatInterval)
+	defer t.Stop()
+	for {
+		select {
+		case <-s.closing:
+			return
+		case <-t.C:
+			size, err := s.DiskSize()
+			if err != nil {
+				s.logger.Printf("Error collecting shard size: %v", err)
+				continue
+			}
+			atomic.StoreInt64(&s.stats.DiskBytes, size)
+		}
+	}
+}
+
 // Shards represents a sortable list of shards.
 type Shards []*Shard
 
@@ -550,8 +723,7 @@ func (a Shards) Swap(i, j int)      { a[i], a[j] = a[j], a[i] }
 type MeasurementFields struct {
 	mu sync.RWMutex
 
-	fields map[string]*Field `json:"fields"`
-	Codec  *FieldCodec
+	fields map[string]*Field
 }
 
 func NewMeasurementFields() *MeasurementFields {
@@ -619,7 +791,6 @@ func (m *MeasurementFields) CreateFieldIfNotExists(name string, typ influxql.Dat
 		Type: typ,
 	}
 	m.fields[name] = f
-	m.Codec = NewFieldCodec(m.fields)
 
 	return nil
 }
@@ -631,6 +802,17 @@ func (m *MeasurementFields) Field(name string) *Field {
 	return f
 }
 
+func (m *MeasurementFields) FieldSet() map[string]influxql.DataType {
+	m.mu.RLock()
+	defer m.mu.RUnlock()
+
+	fields := make(map[string]influxql.DataType)
+	for name, f := range m.fields {
+		fields[name] = f.Type
+	}
+	return fields
+}
+
 // Field represents a series field.
 type Field struct {
 	ID   uint8             `json:"id,omitempty"`
@@ -638,312 +820,43 @@ type Field struct {
 	Type influxql.DataType `json:"type,omitempty"`
 }
 
-// FieldCodec provides encoding and decoding functionality for the fields of a given
-// Measurement. It is a distinct type to avoid locking writes on this node while
-// potentially long-running queries are executing.
-//
-// It is not affected by changes to the Measurement object after codec creation.
-// TODO: this shouldn't be exported. nothing outside the shard should know about field encodings.
-//       However, this is here until tx.go and the engine get refactored into tsdb.
-type FieldCodec struct {
-	fieldsByID   map[uint8]*Field
-	fieldsByName map[string]*Field
-}
-
-// NewFieldCodec returns a FieldCodec for the given Measurement. Must be called with
-// a RLock that protects the Measurement.
-func NewFieldCodec(fields map[string]*Field) *FieldCodec {
-	fieldsByID := make(map[uint8]*Field, len(fields))
-	fieldsByName := make(map[string]*Field, len(fields))
-	for _, f := range fields {
-		fieldsByID[f.ID] = f
-		fieldsByName[f.Name] = f
-	}
-	return &FieldCodec{fieldsByID: fieldsByID, fieldsByName: fieldsByName}
-}
-
-// EncodeFields converts a map of values with string keys to a byte slice of field
-// IDs and values.
-//
-// If a field exists in the codec, but its type is different, an error is returned. If
-// a field is not present in the codec, the system panics.
-func (f *FieldCodec) EncodeFields(values map[string]interface{}) ([]byte, error) {
-	// Allocate byte slice
-	b := make([]byte, 0, 10)
-
-	for k, v := range values {
-		field := f.fieldsByName[k]
-		if field == nil {
-			panic(fmt.Sprintf("field does not exist for %s", k))
-		} else if influxql.InspectDataType(v) != field.Type {
-			return nil, fmt.Errorf("field \"%s\" is type %T, mapped as type %s", k, v, field.Type)
-		}
-
-		var buf []byte
-
-		switch field.Type {
-		case influxql.Float:
-			value := v.(float64)
-			buf = make([]byte, 9)
-			binary.BigEndian.PutUint64(buf[1:9], math.Float64bits(value))
-		case influxql.Integer:
-			var value uint64
-			switch v.(type) {
-			case int:
-				value = uint64(v.(int))
-			case int32:
-				value = uint64(v.(int32))
-			case int64:
-				value = uint64(v.(int64))
-			default:
-				panic(fmt.Sprintf("invalid integer type: %T", v))
-			}
-			buf = make([]byte, 9)
-			binary.BigEndian.PutUint64(buf[1:9], value)
-		case influxql.Boolean:
-			value := v.(bool)
-
-			// Only 1 byte need for a boolean.
-			buf = make([]byte, 2)
-			if value {
-				buf[1] = byte(1)
-			}
-		case influxql.String:
-			value := v.(string)
-			if len(value) > maxStringLength {
-				value = value[:maxStringLength]
-			}
-			// Make a buffer for field ID (1 bytes), the string length (2 bytes), and the string.
-			buf = make([]byte, len(value)+3)
-
-			// Set the string length, then copy the string itself.
-			binary.BigEndian.PutUint16(buf[1:3], uint16(len(value)))
-			for i, c := range []byte(value) {
-				buf[i+3] = byte(c)
-			}
-		default:
-			panic(fmt.Sprintf("unsupported value type during encode fields: %T", v))
-		}
-
-		// Always set the field ID as the leading byte.
-		buf[0] = field.ID
-
-		// Append temp buffer to the end.
-		b = append(b, buf...)
-	}
-
-	return b, nil
-}
-
-// FieldIDByName returns the ID of the field with the given name s.
-// TODO: this shouldn't be exported. remove when tx.go and engine.go get refactored into tsdb
-func (f *FieldCodec) FieldIDByName(s string) (uint8, error) {
-	fi := f.fieldsByName[s]
-	if fi == nil {
-		return 0, ErrFieldNotFound
-	}
-	return fi.ID, nil
-}
-
-// DecodeFields decodes a byte slice into a set of field ids and values.
-func (f *FieldCodec) DecodeFields(b []byte) (map[uint8]interface{}, error) {
-	if len(b) == 0 {
-		return nil, nil
-	}
-
-	// Create a map to hold the decoded data.
-	values := make(map[uint8]interface{}, 0)
-
-	for {
-		if len(b) < 1 {
-			// No more bytes.
-			break
-		}
-
-		// First byte is the field identifier.
-		fieldID := b[0]
-		field := f.fieldsByID[fieldID]
-		if field == nil {
-			// See note in DecodeByID() regarding field-mapping failures.
-			return nil, ErrFieldUnmappedID
-		}
-
-		var value interface{}
-		switch field.Type {
-		case influxql.Float:
-			value = math.Float64frombits(binary.BigEndian.Uint64(b[1:9]))
-			// Move bytes forward.
-			b = b[9:]
-		case influxql.Integer:
-			value = int64(binary.BigEndian.Uint64(b[1:9]))
-			// Move bytes forward.
-			b = b[9:]
-		case influxql.Boolean:
-			if b[1] == 1 {
-				value = true
-			} else {
-				value = false
-			}
-			// Move bytes forward.
-			b = b[2:]
-		case influxql.String:
-			size := binary.BigEndian.Uint16(b[1:3])
-			value = string(b[3 : size+3])
-			// Move bytes forward.
-			b = b[size+3:]
-		default:
-			panic(fmt.Sprintf("unsupported value type during decode fields: %T", f.fieldsByID[fieldID]))
-		}
-
-		values[fieldID] = value
-
-	}
-
-	return values, nil
-}
-
-// DecodeFieldsWithNames decodes a byte slice into a set of field names and values
-// TODO: shouldn't be exported. refactor engine
-func (f *FieldCodec) DecodeFieldsWithNames(b []byte) (map[string]interface{}, error) {
-	fields, err := f.DecodeFields(b)
-	if err != nil {
-		return nil, err
-	}
-	m := make(map[string]interface{})
-	for id, v := range fields {
-		field := f.fieldsByID[id]
-		if field != nil {
-			m[field.Name] = v
-		}
-	}
-	return m, nil
-}
-
-// DecodeByID scans a byte slice for a field with the given ID, converts it to its
-// expected type, and return that value.
-// TODO: shouldn't be exported. refactor engine
-func (f *FieldCodec) DecodeByID(targetID uint8, b []byte) (interface{}, error) {
-	if len(b) == 0 {
-		return 0, ErrFieldNotFound
-	}
-
-	for {
-		if len(b) < 1 {
-			// No more bytes.
-			break
-		}
-		field, ok := f.fieldsByID[b[0]]
-		if !ok {
-			// This can happen, though is very unlikely. If this node receives encoded data, to be written
-			// to disk, and is queried for that data before its metastore is updated, there will be no field
-			// mapping for the data during decode. All this can happen because data is encoded by the node
-			// that first received the write request, not the node that actually writes the data to disk.
-			// So if this happens, the read must be aborted.
-			return 0, ErrFieldUnmappedID
-		}
-
-		var value interface{}
-		switch field.Type {
-		case influxql.Float:
-			// Move bytes forward.
-			value = math.Float64frombits(binary.BigEndian.Uint64(b[1:9]))
-			b = b[9:]
-		case influxql.Integer:
-			value = int64(binary.BigEndian.Uint64(b[1:9]))
-			b = b[9:]
-		case influxql.Boolean:
-			if b[1] == 1 {
-				value = true
-			} else {
-				value = false
-			}
-			// Move bytes forward.
-			b = b[2:]
-		case influxql.String:
-			size := binary.BigEndian.Uint16(b[1:3])
-			value = string(b[3 : 3+size])
-			// Move bytes forward.
-			b = b[size+3:]
-		default:
-			panic(fmt.Sprintf("unsupported value type during decode by id: %T", field.Type))
-		}
-
-		if field.ID == targetID {
-			return value, nil
-		}
-	}
-
-	return 0, ErrFieldNotFound
-}
-
-// DecodeByName scans a byte slice for a field with the given name, converts it to its
-// expected type, and return that value.
-func (f *FieldCodec) DecodeByName(name string, b []byte) (interface{}, error) {
-	fi := f.FieldByName(name)
-	if fi == nil {
-		return 0, ErrFieldNotFound
-	}
-	return f.DecodeByID(fi.ID, b)
-}
-
-// Fields returns a unsorted list of the codecs fields.
-func (f *FieldCodec) Fields() []*Field {
-	a := make([]*Field, 0, len(f.fieldsByID))
-	for _, f := range f.fieldsByID {
-		a = append(a, f)
-	}
-	return a
-}
-
-// FieldByName returns the field by its name. It will return a nil if not found
-func (f *FieldCodec) FieldByName(name string) *Field {
-	return f.fieldsByName[name]
-}
-
 // shardIteratorCreator creates iterators for a local shard.
 // This simply wraps the shard so that Close() does not close the underlying shard.
 type shardIteratorCreator struct {
-	sh *Shard
+	sh         *Shard
+	maxSeriesN int
 }
 
 func (ic *shardIteratorCreator) Close() error { return nil }
 
 func (ic *shardIteratorCreator) CreateIterator(opt influxql.IteratorOptions) (influxql.Iterator, error) {
-	return ic.sh.CreateIterator(opt)
+	itr, err := ic.sh.CreateIterator(opt)
+	if err != nil {
+		return nil, err
+	} else if itr == nil {
+		return nil, nil
+	}
+
+	// Enforce series limit at creation time.
+	if ic.maxSeriesN > 0 {
+		stats := itr.Stats()
+		if stats.SeriesN > ic.maxSeriesN {
+			itr.Close()
+			return nil, fmt.Errorf("max select series count exceeded: %d series", stats.SeriesN)
+		}
+	}
+
+	return itr, nil
 }
-func (ic *shardIteratorCreator) FieldDimensions(sources influxql.Sources) (fields, dimensions map[string]struct{}, err error) {
+func (ic *shardIteratorCreator) FieldDimensions(sources influxql.Sources) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error) {
 	return ic.sh.FieldDimensions(sources)
 }
-func (ic *shardIteratorCreator) SeriesKeys(opt influxql.IteratorOptions) (influxql.SeriesList, error) {
-	return ic.sh.SeriesKeys(opt)
-}
 func (ic *shardIteratorCreator) ExpandSources(sources influxql.Sources) (influxql.Sources, error) {
 	return ic.sh.ExpandSources(sources)
 }
 
 func NewFieldKeysIterator(sh *Shard, opt influxql.IteratorOptions) (influxql.Iterator, error) {
-	fn := func(m *Measurement) []string {
-		keys := m.FieldNames()
-		sort.Strings(keys)
-		return keys
-	}
-	return newMeasurementKeysIterator(sh, fn, opt)
-}
-
-// MeasurementIterator represents a string iterator that emits all measurement names in a shard.
-type MeasurementIterator struct {
-	mms    Measurements
-	source *influxql.Measurement
-}
-
-// NewMeasurementIterator returns a new instance of MeasurementIterator.
-func NewMeasurementIterator(sh *Shard, opt influxql.IteratorOptions) (*MeasurementIterator, error) {
-	itr := &MeasurementIterator{}
-
-	// Extract source.
-	if len(opt.Sources) > 0 {
-		itr.source, _ = opt.Sources[0].(*influxql.Measurement)
-	}
+	itr := &fieldKeysIterator{sh: sh}
 
 	// Retrieve measurements from shard. Filter if condition specified.
 	if opt.Condition == nil {
@@ -962,23 +875,65 @@ func NewMeasurementIterator(sh *Shard, opt influxql.IteratorOptions) (*Measureme
 	return itr, nil
 }
 
+// fieldKeysIterator iterates over measurements and gets field keys from each measurement.
+type fieldKeysIterator struct {
+	sh  *Shard
+	mms Measurements // remaining measurements
+	buf struct {
+		mm     *Measurement // current measurement
+		fields []Field      // current measurement's fields
+	}
+}
+
 // Stats returns stats about the points processed.
-func (itr *MeasurementIterator) Stats() influxql.IteratorStats { return influxql.IteratorStats{} }
+func (itr *fieldKeysIterator) Stats() influxql.IteratorStats { return influxql.IteratorStats{} }
 
 // Close closes the iterator.
-func (itr *MeasurementIterator) Close() error { return nil }
+func (itr *fieldKeysIterator) Close() error { return nil }
 
-// Next emits the next measurement name.
-func (itr *MeasurementIterator) Next() (*influxql.FloatPoint, error) {
-	if len(itr.mms) == 0 {
-		return nil, nil
+// Next emits the next tag key name.
+func (itr *fieldKeysIterator) Next() (*influxql.FloatPoint, error) {
+	for {
+		// If there are no more keys then move to the next measurements.
+		if len(itr.buf.fields) == 0 {
+			if len(itr.mms) == 0 {
+				return nil, nil
+			}
+
+			itr.buf.mm = itr.mms[0]
+			mf := itr.sh.engine.MeasurementFields(itr.buf.mm.Name)
+			if mf != nil {
+				fset := mf.FieldSet()
+				if len(fset) == 0 {
+					itr.mms = itr.mms[1:]
+					continue
+				}
+
+				keys := make([]string, 0, len(fset))
+				for k := range fset {
+					keys = append(keys, k)
+				}
+				sort.Strings(keys)
+
+				itr.buf.fields = make([]Field, len(keys))
+				for i, name := range keys {
+					itr.buf.fields[i] = Field{Name: name, Type: fset[name]}
+				}
+			}
+			itr.mms = itr.mms[1:]
+			continue
+		}
+
+		// Return next key.
+		field := itr.buf.fields[0]
+		p := &influxql.FloatPoint{
+			Name: itr.buf.mm.Name,
+			Aux:  []interface{}{field.Name, field.Type.String()},
+		}
+		itr.buf.fields = itr.buf.fields[1:]
+
+		return p, nil
 	}
-	mm := itr.mms[0]
-	itr.mms = itr.mms[1:]
-	return &influxql.FloatPoint{
-		Name: "measurements",
-		Aux:  []interface{}{mm.Name},
-	}, nil
 }
 
 // seriesIterator emits series ids.
@@ -1050,7 +1005,7 @@ func (itr *seriesIterator) Next() (*influxql.FloatPoint, error) {
 
 		// Write auxiliary fields.
 		for i, f := range itr.opt.Aux {
-			switch f {
+			switch f.Val {
 			case "key":
 				itr.point.Aux[i] = key
 			}
@@ -1094,125 +1049,6 @@ func NewTagKeysIterator(sh *Shard, opt influxql.IteratorOptions) (influxql.Itera
 	return newMeasurementKeysIterator(sh, fn, opt)
 }
 
-// tagValuesIterator emits key/tag values
-type tagValuesIterator struct {
-	series []*Series // remaining series
-	keys   []string  // tag keys to select from a series
-	fields []string  // fields to emit (key or value)
-	buf    struct {
-		s    *Series  // current series
-		keys []string // current tag's keys
-	}
-}
-
-// NewTagValuesIterator returns a new instance of TagValuesIterator.
-func NewTagValuesIterator(sh *Shard, opt influxql.IteratorOptions) (influxql.Iterator, error) {
-	if opt.Condition == nil {
-		return nil, errors.New("a condition is required")
-	}
-
-	mms, ok, err := sh.index.measurementsByExpr(opt.Condition)
-	if err != nil {
-		return nil, err
-	} else if !ok {
-		mms = sh.index.Measurements()
-		sort.Sort(mms)
-	}
-
-	filterExpr := influxql.CloneExpr(opt.Condition)
-	filterExpr = influxql.RewriteExpr(filterExpr, func(e influxql.Expr) influxql.Expr {
-		switch e := e.(type) {
-		case *influxql.BinaryExpr:
-			switch e.Op {
-			case influxql.EQ, influxql.NEQ, influxql.EQREGEX, influxql.NEQREGEX:
-				tag, ok := e.LHS.(*influxql.VarRef)
-				if !ok || strings.HasPrefix(tag.Val, "_") {
-					return nil
-				}
-			}
-		}
-		return e
-	})
-
-	var series []*Series
-	keys := newStringSet()
-	for _, mm := range mms {
-		ss, ok, err := mm.tagKeysByExpr(opt.Condition)
-		if err != nil {
-			return nil, err
-		} else if !ok {
-			keys.add(mm.TagKeys()...)
-		} else {
-			keys = keys.union(ss)
-		}
-
-		ids, err := mm.seriesIDsAllOrByExpr(filterExpr)
-		if err != nil {
-			return nil, err
-		}
-
-		for _, id := range ids {
-			series = append(series, mm.SeriesByID(id))
-		}
-	}
-
-	return &tagValuesIterator{
-		series: series,
-		keys:   keys.list(),
-		fields: opt.Aux,
-	}, nil
-}
-
-// Stats returns stats about the points processed.
-func (itr *tagValuesIterator) Stats() influxql.IteratorStats { return influxql.IteratorStats{} }
-
-// Close closes the iterator.
-func (itr *tagValuesIterator) Close() error { return nil }
-
-// Next emits the next point in the iterator.
-func (itr *tagValuesIterator) Next() (*influxql.FloatPoint, error) {
-	for {
-		// If there are no more values then move to the next key.
-		if len(itr.buf.keys) == 0 {
-			if len(itr.series) == 0 {
-				return nil, nil
-			}
-
-			itr.buf.s = itr.series[0]
-			itr.buf.keys = itr.keys
-			itr.series = itr.series[1:]
-			continue
-		}
-
-		key := itr.buf.keys[0]
-		value, ok := itr.buf.s.Tags[key]
-		if !ok {
-			itr.buf.keys = itr.buf.keys[1:]
-			continue
-		}
-
-		// Prepare auxiliary fields.
-		auxFields := make([]interface{}, len(itr.fields))
-		for i, f := range itr.fields {
-			switch f {
-			case "_tagKey":
-				auxFields[i] = key
-			case "value":
-				auxFields[i] = value
-			}
-		}
-
-		// Return next key.
-		p := &influxql.FloatPoint{
-			Name: itr.buf.s.measurement.Name,
-			Aux:  auxFields,
-		}
-		itr.buf.keys = itr.buf.keys[1:]
-
-		return p, nil
-	}
-}
-
 // measurementKeyFunc is the function called by measurementKeysIterator.
 type measurementKeyFunc func(m *Measurement) []string
 
@@ -1277,33 +1113,3 @@ func (itr *measurementKeysIterator) Next() (*influxql.FloatPoint, error) {
 		return p, nil
 	}
 }
-
-// IsNumeric returns whether a given aggregate can only be run on numeric fields.
-func IsNumeric(c *influxql.Call) bool {
-	switch c.Name {
-	case "count", "first", "last", "distinct":
-		return false
-	default:
-		return true
-	}
-}
-
-// mustMarshal encodes a value to JSON.
-// This will panic if an error occurs. This should only be used internally when
-// an invalid marshal will cause corruption and a panic is appropriate.
-func mustMarshalJSON(v interface{}) []byte {
-	b, err := json.Marshal(v)
-	if err != nil {
-		panic("marshal: " + err.Error())
-	}
-	return b
-}
-
-// mustUnmarshalJSON decodes a value from JSON.
-// This will panic if an error occurs. This should only be used internally when
-// an invalid unmarshal will cause corruption and a panic is appropriate.
-func mustUnmarshalJSON(b []byte, v interface{}) {
-	if err := json.Unmarshal(b, v); err != nil {
-		panic("unmarshal: " + err.Error())
-	}
-}
