@@ -1,6 +1,7 @@
 package tsdb_test
 
 import (
+	"fmt"
 	"io/ioutil"
 	"os"
 	"path"
@@ -96,6 +97,59 @@ func TestShardWriteAndIndex(t *testing.T) {
 	}
 }
 
+func TestMaxSeriesLimit(t *testing.T) {
+	tmpDir, _ := ioutil.TempDir("", "shard_test")
+	defer os.RemoveAll(tmpDir)
+	tmpShard := path.Join(tmpDir, "shard")
+	tmpWal := path.Join(tmpDir, "wal")
+
+	index := tsdb.NewDatabaseIndex("db")
+	opts := tsdb.NewEngineOptions()
+	opts.Config.WALDir = filepath.Join(tmpDir, "wal")
+	opts.Config.MaxSeriesPerDatabase = 1000
+
+	sh := tsdb.NewShard(1, index, tmpShard, tmpWal, opts)
+
+	if err := sh.Open(); err != nil {
+		t.Fatalf("error opening shard: %s", err.Error())
+	}
+
+	// Writing 1K series should succeed.
+	points := []models.Point{}
+
+	for i := 0; i < 1000; i++ {
+		pt := models.MustNewPoint(
+			"cpu",
+			map[string]string{"host": fmt.Sprintf("server%d", i)},
+			map[string]interface{}{"value": 1.0},
+			time.Unix(1, 2),
+		)
+		points = append(points, pt)
+	}
+
+	err := sh.WritePoints(points)
+	if err != nil {
+		t.Fatalf(err.Error())
+	}
+
+	// Writing one more series should exceed the series limit.
+	pt := models.MustNewPoint(
+		"cpu",
+		map[string]string{"host": "server9999"},
+		map[string]interface{}{"value": 1.0},
+		time.Unix(1, 2),
+	)
+
+	err = sh.WritePoints([]models.Point{pt})
+	if err == nil {
+		t.Fatal("expected error")
+	} else if err.Error() != "max series per database exceeded: cpu,host=server9999" {
+		t.Fatalf("unexpected error messag:\n\texp = max series per database exceeded: cpu,host=server9999\n\tgot = %s", err.Error())
+	}
+
+	sh.Close()
+}
+
 func TestShardWriteAddNewField(t *testing.T) {
 	tmpDir, _ := ioutil.TempDir("", "shard_test")
 	defer os.RemoveAll(tmpDir)
@@ -219,12 +273,16 @@ cpu,host=serverB,region=uswest value=25  0
 	// Create iterator.
 	itr, err := sh.CreateIterator(influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr(`value`),
-		Aux:        []string{"val2"},
+		Aux:        []influxql.VarRef{{Val: "val2"}},
 		Dimensions: []string{"host"},
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
-		Ascending:  true,
-		StartTime:  influxql.MinTime,
-		EndTime:    influxql.MaxTime,
+		Sources: []influxql.Source{&influxql.Measurement{
+			Name:            "cpu",
+			Database:        "db0",
+			RetentionPolicy: "rp0",
+		}},
+		Ascending: true,
+		StartTime: influxql.MinTime,
+		EndTime:   influxql.MaxTime,
 	})
 	if err != nil {
 		t.Fatal(err)
@@ -295,12 +353,16 @@ cpu,host=serverB,region=uswest value=25  0
 	// Create iterator.
 	itr, err := sh.CreateIterator(influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr(`value`),
-		Aux:        []string{"val2"},
+		Aux:        []influxql.VarRef{{Val: "val2"}},
 		Dimensions: []string{"host"},
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
-		Ascending:  false,
-		StartTime:  influxql.MinTime,
-		EndTime:    influxql.MaxTime,
+		Sources: []influxql.Source{&influxql.Measurement{
+			Name:            "cpu",
+			Database:        "db0",
+			RetentionPolicy: "rp0",
+		}},
+		Ascending: false,
+		StartTime: influxql.MinTime,
+		EndTime:   influxql.MaxTime,
 	})
 	if err != nil {
 		t.Fatal(err)
@@ -346,12 +408,165 @@ cpu,host=serverB,region=uswest value=25  0
 	}
 }
 
+func TestShard_Disabled_WriteQuery(t *testing.T) {
+	sh := NewShard()
+	if err := sh.Open(); err != nil {
+		t.Fatal(err)
+	}
+	defer sh.Close()
+
+	sh.SetEnabled(false)
+
+	pt := models.MustNewPoint(
+		"cpu",
+		map[string]string{"host": "server"},
+		map[string]interface{}{"value": 1.0},
+		time.Unix(1, 2),
+	)
+
+	err := sh.WritePoints([]models.Point{pt})
+	if err == nil {
+		t.Fatalf("expected shard disabled error")
+	}
+	if err != tsdb.ErrShardDisabled {
+		t.Fatalf(err.Error())
+	}
+
+	_, got := sh.CreateIterator(influxql.IteratorOptions{})
+	if err == nil {
+		t.Fatalf("expected shard disabled error")
+	}
+	if exp := tsdb.ErrShardDisabled; got != exp {
+		t.Fatalf("got %v, expected %v", got, exp)
+	}
+
+	sh.SetEnabled(true)
+
+	err = sh.WritePoints([]models.Point{pt})
+	if err != nil {
+		t.Fatalf("unexpected error: %v", err)
+	}
+
+	if _, err = sh.CreateIterator(influxql.IteratorOptions{}); err != nil {
+		t.Fatalf("unexpected error: %v", got)
+	}
+}
+
 func BenchmarkWritePoints_NewSeries_1K(b *testing.B)   { benchmarkWritePoints(b, 38, 3, 3, 1) }
 func BenchmarkWritePoints_NewSeries_100K(b *testing.B) { benchmarkWritePoints(b, 32, 5, 5, 1) }
 func BenchmarkWritePoints_NewSeries_250K(b *testing.B) { benchmarkWritePoints(b, 80, 5, 5, 1) }
 func BenchmarkWritePoints_NewSeries_500K(b *testing.B) { benchmarkWritePoints(b, 160, 5, 5, 1) }
 func BenchmarkWritePoints_NewSeries_1M(b *testing.B)   { benchmarkWritePoints(b, 320, 5, 5, 1) }
 
+// Fix measurement and tag key cardinalities and vary tag value cardinality
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_100_TagValues(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 100, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_500_TagValues(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 500, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_1000_TagValues(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 1000, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_5000_TagValues(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 5000, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_10000_TagValues(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 10000, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_50000_TagValues(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 50000, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_100000_TagValues(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 100000, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_500000_TagValues(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 500000, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_1000000_TagValues(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 1000000, 1)
+}
+
+// Fix tag key and tag values cardinalities and vary measurement cardinality
+func BenchmarkWritePoints_NewSeries_100_Measurements_1_TagKey_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 100, 1, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_500_Measurements_1_TagKey_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 500, 1, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1000_Measurement_1_TagKey_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1000, 1, 1, 1)
+}
+
+func BenchmarkWritePoints_NewSeries_5000_Measurement_1_TagKey_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 5000, 1, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_10000_Measurement_1_TagKey_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 10000, 1, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_50000_Measurement_1_TagKey_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 50000, 1, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_100000_Measurement_1_TagKey_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 100000, 1, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_500000_Measurement_1_TagKey_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 500000, 1, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1000000_Measurement_1_TagKey_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1000000, 1, 1, 1)
+}
+
+// Fix measurement and tag values cardinalities and vary tag key cardinality
+func BenchmarkWritePoints_NewSeries_1_Measurement_2_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<1, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurements_4_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<2, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurements_8_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<3, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_16_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<4, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_32_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<5, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_64_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<6, 1, 1)
+
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_128_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<7, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_256_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<8, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_512_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<9, 1, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_1024_TagKeys_1_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1<<10, 1, 1)
+}
+
+// Fix series cardinality and vary tag keys and value cardinalities
+func BenchmarkWritePoints_NewSeries_1_Measurement_1_TagKey_65536_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 1, 1<<16, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_2_TagKeys_256_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 2, 1<<8, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_4_TagKeys_16_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 4, 1<<4, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_8_TagKeys_4_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 8, 1<<2, 1)
+}
+func BenchmarkWritePoints_NewSeries_1_Measurement_16_TagKeys_2_TagValue(b *testing.B) {
+	benchmarkWritePoints(b, 1, 16, 1<<1, 1)
+}
+
 func BenchmarkWritePoints_ExistingSeries_1K(b *testing.B) {
 	benchmarkWritePointsExistingSeries(b, 38, 3, 3, 1)
 }
@@ -494,8 +709,8 @@ func NewShard() *Shard {
 	return &Shard{
 		Shard: tsdb.NewShard(0,
 			tsdb.NewDatabaseIndex("db"),
-			filepath.Join(path, "data"),
-			filepath.Join(path, "wal"),
+			filepath.Join(path, "data", "db0", "rp0", "1"),
+			filepath.Join(path, "wal", "db0", "rp0", "1"),
 			opt,
 		),
 		path: path,
