@@ -8,6 +8,7 @@ import (
 	"log"
 	"os"
 	"path/filepath"
+	"runtime"
 	"sort"
 	"strconv"
 	"strings"
@@ -16,6 +17,7 @@ import (
 
 	"github.com/influxdata/influxdb/influxql"
 	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/pkg/limiter"
 )
 
 var (
@@ -25,10 +27,6 @@ var (
 	ErrStoreClosed = fmt.Errorf("store is closed")
 )
 
-const (
-	maintenanceCheckInterval = time.Minute
-)
-
 // Store manages shards and indexes for databases.
 type Store struct {
 	mu   sync.RWMutex
@@ -54,7 +52,6 @@ type Store struct {
 // The returned store must be initialized by calling Open before using it.
 func NewStore(path string) *Store {
 	opts := NewEngineOptions()
-	opts.Config = NewConfig()
 
 	return &Store{
 		path:          path,
@@ -64,14 +61,36 @@ func NewStore(path string) *Store {
 	}
 }
 
-// SetLogOutput sets the writer to which all logs are written. It must not be
-// called after Open is called.
+// SetLogOutput sets the writer to which all logs are written. It is safe for
+// concurrent use.
 func (s *Store) SetLogOutput(w io.Writer) {
-	s.Logger = log.New(w, "[store] ", log.LstdFlags)
-	s.logOutput = w
+	s.Logger.SetOutput(w)
 	for _, s := range s.shards {
 		s.SetLogOutput(w)
 	}
+
+	s.mu.Lock()
+	s.logOutput = w
+	s.mu.Unlock()
+}
+
+func (s *Store) Statistics(tags map[string]string) []models.Statistic {
+	var statistics []models.Statistic
+
+	s.mu.RLock()
+	indexes := make([]models.Statistic, 0, len(s.databaseIndexes))
+	for _, dbi := range s.databaseIndexes {
+		indexes = append(indexes, dbi.Statistics(tags)...)
+	}
+	shards := s.shardsSlice()
+	s.mu.RUnlock()
+
+	for _, shard := range shards {
+		statistics = append(statistics, shard.Statistics(tags)...)
+	}
+
+	statistics = append(statistics, indexes...)
+	return statistics
 }
 
 // Path returns the store's root path.
@@ -131,7 +150,7 @@ func (s *Store) loadShards() error {
 		err error
 	}
 
-	throttle := newthrottle(4)
+	t := limiter.NewFixed(runtime.GOMAXPROCS(0))
 
 	resC := make(chan *res)
 	var n int
@@ -157,8 +176,8 @@ func (s *Store) loadShards() error {
 			for _, sh := range shards {
 				n++
 				go func(index *DatabaseIndex, db, rp, sh string) {
-					throttle.take()
-					defer throttle.release()
+					t.Take()
+					defer t.Release()
 
 					start := time.Now()
 					path := filepath.Join(s.path, db, rp, sh)
@@ -263,7 +282,7 @@ func (s *Store) ShardN() int {
 }
 
 // CreateShard creates a shard with the given id and retention policy on a database.
-func (s *Store) CreateShard(database, retentionPolicy string, shardID uint64) error {
+func (s *Store) CreateShard(database, retentionPolicy string, shardID uint64, enabled bool) error {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
@@ -299,6 +318,8 @@ func (s *Store) CreateShard(database, retentionPolicy string, shardID uint64) er
 	path := filepath.Join(s.path, database, retentionPolicy, strconv.FormatUint(shardID, 10))
 	shard := NewShard(shardID, db, path, walPath, s.EngineOptions)
 	shard.SetLogOutput(s.logOutput)
+	shard.EnableOnOpen = enabled
+
 	if err := shard.Open(); err != nil {
 		return err
 	}
@@ -308,22 +329,39 @@ func (s *Store) CreateShard(database, retentionPolicy string, shardID uint64) er
 	return nil
 }
 
-// DeleteShard removes a shard from disk.
-func (s *Store) DeleteShard(shardID uint64) error {
-	s.mu.Lock()
-	defer s.mu.Unlock()
-	return s.deleteShard(shardID)
+// CreateShardSnapShot will create a hard link to the underlying shard and return a path
+// The caller is responsible for cleaning up (removing) the file path returned
+func (s *Store) CreateShardSnapshot(id uint64) (string, error) {
+	sh := s.Shard(id)
+	if sh == nil {
+		return "", ErrShardNotFound
+	}
+
+	return sh.CreateSnapshot()
 }
 
-// deleteShard removes a shard from disk. Callers of deleteShard need
-// to handle locks appropriately.
-func (s *Store) deleteShard(shardID uint64) error {
-	// ensure shard exists
-	sh, ok := s.shards[shardID]
-	if !ok {
+// SetShardEnabled enables or disables a shard for read and writes
+func (s *Store) SetShardEnabled(shardID uint64, enabled bool) error {
+	sh := s.Shard(shardID)
+	if sh == nil {
+		return ErrShardNotFound
+	}
+	sh.SetEnabled(enabled)
+	return nil
+}
+
+// DeleteShard removes a shard from disk.
+func (s *Store) DeleteShard(shardID uint64) error {
+	sh := s.Shard(shardID)
+	if sh == nil {
 		return nil
 	}
 
+	// Remove the shard from the database indexes before closing the shard.
+	// Closing the shard will do this as well, but it will unload it while
+	// the shard is locked which can block stats collection and other calls.
+	sh.UnloadIndex()
+
 	if err := sh.Close(); err != nil {
 		return err
 	}
@@ -336,32 +374,41 @@ func (s *Store) deleteShard(shardID uint64) error {
 		return err
 	}
 
+	s.mu.Lock()
 	delete(s.shards, shardID)
+	s.mu.Unlock()
+
 	return nil
 }
 
 // ShardIteratorCreator returns an iterator creator for a shard.
-func (s *Store) ShardIteratorCreator(id uint64) influxql.IteratorCreator {
+func (s *Store) ShardIteratorCreator(id uint64, opt *influxql.SelectOptions) influxql.IteratorCreator {
 	sh := s.Shard(id)
 	if sh == nil {
 		return nil
 	}
-	return &shardIteratorCreator{sh: sh}
+	return &shardIteratorCreator{
+		sh:         sh,
+		maxSeriesN: opt.MaxSeriesN,
+	}
 }
 
 // DeleteDatabase will close all shards associated with a database and remove the directory and files from disk.
 func (s *Store) DeleteDatabase(name string) error {
-	s.mu.Lock()
-	defer s.mu.Unlock()
+	s.mu.RLock()
+	shards := s.filterShards(func(sh *Shard) bool {
+		return sh.database == name
+	})
+	s.mu.RUnlock()
 
-	// Close and delete all shards on the database.
-	for shardID, sh := range s.shards {
-		if sh.database == name {
-			// Delete the shard from disk.
-			if err := s.deleteShard(shardID); err != nil {
-				return err
-			}
+	if err := s.walkShards(shards, func(sh *Shard) error {
+		if sh.database != name {
+			return nil
 		}
+
+		return sh.Close()
+	}); err != nil {
+		return err
 	}
 
 	if err := os.RemoveAll(filepath.Join(s.path, name)); err != nil {
@@ -371,7 +418,13 @@ func (s *Store) DeleteDatabase(name string) error {
 		return err
 	}
 
+	s.mu.Lock()
+	for _, sh := range shards {
+		delete(s.shards, sh.id)
+	}
 	delete(s.databaseIndexes, name)
+	s.mu.Unlock()
+
 	return nil
 }
 
@@ -379,18 +432,22 @@ func (s *Store) DeleteDatabase(name string) error {
 // provided retention policy, remove the retention policy directories on
 // both the DB and WAL, and remove all shard files from disk.
 func (s *Store) DeleteRetentionPolicy(database, name string) error {
-	s.mu.Lock()
-	defer s.mu.Unlock()
+	s.mu.RLock()
+	shards := s.filterShards(func(sh *Shard) bool {
+		return sh.database == database && sh.retentionPolicy == name
+	})
+	s.mu.RUnlock()
 
 	// Close and delete all shards under the retention policy on the
 	// database.
-	for shardID, sh := range s.shards {
-		if sh.database == database && sh.retentionPolicy == name {
-			// Delete the shard from disk.
-			if err := s.deleteShard(shardID); err != nil {
-				return err
-			}
+	if err := s.walkShards(shards, func(sh *Shard) error {
+		if sh.database != database || sh.retentionPolicy != name {
+			return nil
 		}
+
+		return sh.Close()
+	}); err != nil {
+		return err
 	}
 
 	// Remove the rentention policy folder.
@@ -399,16 +456,24 @@ func (s *Store) DeleteRetentionPolicy(database, name string) error {
 	}
 
 	// Remove the retention policy folder from the the WAL.
-	return os.RemoveAll(filepath.Join(s.EngineOptions.Config.WALDir, database, name))
+	if err := os.RemoveAll(filepath.Join(s.EngineOptions.Config.WALDir, database, name)); err != nil {
+		return err
+	}
+
+	s.mu.Lock()
+	for _, sh := range shards {
+		delete(s.shards, sh.id)
+	}
+	s.mu.Unlock()
+	return nil
 }
 
 // DeleteMeasurement removes a measurement and all associated series from a database.
 func (s *Store) DeleteMeasurement(database, name string) error {
-	s.mu.Lock()
-	defer s.mu.Unlock()
-
 	// Find the database.
+	s.mu.RLock()
 	db := s.databaseIndexes[database]
+	s.mu.RUnlock()
 	if db == nil {
 		return nil
 	}
@@ -419,21 +484,74 @@ func (s *Store) DeleteMeasurement(database, name string) error {
 		return influxql.ErrMeasurementNotFound(name)
 	}
 
+	seriesKeys := m.SeriesKeys()
+
+	s.mu.RLock()
+	shards := s.filterShards(func(sh *Shard) bool {
+		return sh.database == database
+	})
+	s.mu.RUnlock()
+
+	if err := s.walkShards(shards, func(sh *Shard) error {
+		if err := sh.DeleteMeasurement(m.Name, seriesKeys); err != nil {
+			return err
+		}
+		return nil
+	}); err != nil {
+		return err
+	}
+
 	// Remove measurement from index.
 	db.DropMeasurement(m.Name)
 
-	// Remove underlying data.
-	for _, sh := range s.shards {
-		if sh.database != database {
-			continue
-		}
+	return nil
+}
 
-		if err := sh.DeleteMeasurement(m.Name, m.SeriesKeys()); err != nil {
-			return err
+// filterShards returns a slice of shards where fn returns true
+// for the shard.
+func (s *Store) filterShards(fn func(sh *Shard) bool) []*Shard {
+	shards := make([]*Shard, 0, len(s.shards))
+	for _, sh := range s.shards {
+		if fn(sh) {
+			shards = append(shards, sh)
 		}
 	}
+	return shards
+}
 
-	return nil
+// walkShards apply a function to each shard in parallel.  If any of the
+// functions return an error, the first error is returned.
+func (s *Store) walkShards(shards []*Shard, fn func(sh *Shard) error) error {
+	// struct to hold the result of opening each reader in a goroutine
+	type res struct {
+		err error
+	}
+
+	resC := make(chan res)
+	var n int
+
+	for _, sh := range shards {
+		n++
+
+		go func(sh *Shard) {
+			if err := fn(sh); err != nil {
+				resC <- res{err: fmt.Errorf("shard %d: %s", sh.id, err)}
+				return
+			}
+
+			resC <- res{}
+		}(sh)
+	}
+
+	var err error
+	for i := 0; i < n; i++ {
+		res := <-resC
+		if res.err != nil {
+			err = res.err
+		}
+	}
+	close(resC)
+	return err
 }
 
 // ShardIDs returns a slice of all ShardIDs under management.
@@ -521,6 +639,22 @@ func (s *Store) BackupShard(id uint64, since time.Time, w io.Writer) error {
 	return shard.engine.Backup(w, path, since)
 }
 
+// RestoreShard restores a backup from r to a given shard.
+// This will only overwrite files included in the backup.
+func (s *Store) RestoreShard(id uint64, r io.Reader) error {
+	shard := s.Shard(id)
+	if shard == nil {
+		return fmt.Errorf("shard %d doesn't exist on this server", id)
+	}
+
+	path, err := relativePath(s.path, shard.path)
+	if err != nil {
+		return err
+	}
+
+	return shard.Restore(r, path)
+}
+
 // ShardRelativePath will return the relative path to the shard. i.e. <database>/<retention>/<id>
 func (s *Store) ShardRelativePath(id uint64) (string, error) {
 	shard := s.Shard(id)
@@ -605,9 +739,15 @@ func (s *Store) deleteSeries(database string, seriesKeys []string, min, max int6
 		return influxql.ErrDatabaseNotFound(database)
 	}
 
-	for _, sh := range s.shards {
+	s.mu.RLock()
+	shards := s.filterShards(func(sh *Shard) bool {
+		return sh.database == database
+	})
+	s.mu.RUnlock()
+
+	return s.walkShards(shards, func(sh *Shard) error {
 		if sh.database != database {
-			continue
+			return nil
 		}
 		if err := sh.DeleteSeriesRange(seriesKeys, min, max); err != nil {
 			return err
@@ -625,9 +765,8 @@ func (s *Store) deleteSeries(database string, seriesKeys []string, min, max int6
 				db.UnassignShard(k, sh.id)
 			}
 		}
-	}
-
-	return nil
+		return nil
+	})
 }
 
 // ExpandSources expands sources against all local shards.
@@ -637,8 +776,8 @@ func (s *Store) ExpandSources(sources influxql.Sources) (influxql.Sources, error
 
 // IteratorCreators returns a set of all local shards as iterator creators.
 func (s *Store) IteratorCreators() influxql.IteratorCreators {
-	s.mu.Lock()
-	defer s.mu.Unlock()
+	s.mu.RLock()
+	defer s.mu.RUnlock()
 
 	a := make(influxql.IteratorCreators, 0, len(s.shards))
 	for _, sh := range s.shards {
@@ -647,12 +786,12 @@ func (s *Store) IteratorCreators() influxql.IteratorCreators {
 	return a
 }
 
-func (s *Store) IteratorCreator(shards []uint64) (influxql.IteratorCreator, error) {
+func (s *Store) IteratorCreator(shards []uint64, opt *influxql.SelectOptions) (influxql.IteratorCreator, error) {
 	// Generate iterators for each node.
 	ics := make([]influxql.IteratorCreator, 0)
 	if err := func() error {
 		for _, id := range shards {
-			ic := s.ShardIteratorCreator(id)
+			ic := s.ShardIteratorCreator(id, opt)
 			if ic == nil {
 				continue
 			}
@@ -689,6 +828,155 @@ func (s *Store) WriteToShard(shardID uint64, points []models.Point) error {
 	return sh.WritePoints(points)
 }
 
+func (s *Store) Measurements(database string, cond influxql.Expr) ([]string, error) {
+	dbi := s.DatabaseIndex(database)
+	if dbi == nil {
+		return nil, nil
+	}
+
+	// Retrieve measurements from database index. Filter if condition specified.
+	var mms Measurements
+	if cond == nil {
+		mms = dbi.Measurements()
+	} else {
+		var err error
+		mms, _, err = dbi.MeasurementsByExpr(cond)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	// Sort measurements by name.
+	sort.Sort(mms)
+
+	measurements := make([]string, len(mms))
+	for i, m := range mms {
+		measurements[i] = m.Name
+	}
+
+	return measurements, nil
+}
+
+type TagValues struct {
+	Measurement string
+	Values      []KeyValue
+}
+
+func (s *Store) TagValues(database string, cond influxql.Expr) ([]TagValues, error) {
+	if cond == nil {
+		return nil, errors.New("a condition is required")
+	}
+
+	dbi := s.DatabaseIndex(database)
+	if dbi == nil {
+		return nil, nil
+	}
+
+	measurementExpr := influxql.CloneExpr(cond)
+	measurementExpr = influxql.Reduce(influxql.RewriteExpr(measurementExpr, func(e influxql.Expr) influxql.Expr {
+		switch e := e.(type) {
+		case *influxql.BinaryExpr:
+			switch e.Op {
+			case influxql.EQ, influxql.NEQ, influxql.EQREGEX, influxql.NEQREGEX:
+				tag, ok := e.LHS.(*influxql.VarRef)
+				if !ok || tag.Val != "_name" {
+					return nil
+				}
+			}
+		}
+		return e
+	}), nil)
+
+	mms, ok, err := dbi.MeasurementsByExpr(measurementExpr)
+	if err != nil {
+		return nil, err
+	} else if !ok {
+		mms = dbi.Measurements()
+		sort.Sort(mms)
+	}
+
+	// If there are no measurements, return immediately.
+	if len(mms) == 0 {
+		return nil, nil
+	}
+
+	filterExpr := influxql.CloneExpr(cond)
+	filterExpr = influxql.Reduce(influxql.RewriteExpr(filterExpr, func(e influxql.Expr) influxql.Expr {
+		switch e := e.(type) {
+		case *influxql.BinaryExpr:
+			switch e.Op {
+			case influxql.EQ, influxql.NEQ, influxql.EQREGEX, influxql.NEQREGEX:
+				tag, ok := e.LHS.(*influxql.VarRef)
+				if !ok || strings.HasPrefix(tag.Val, "_") {
+					return nil
+				}
+			}
+		}
+		return e
+	}), nil)
+
+	tagValues := make([]TagValues, len(mms))
+	for i, mm := range mms {
+		tagValues[i].Measurement = mm.Name
+
+		ids, err := mm.SeriesIDsAllOrByExpr(filterExpr)
+		if err != nil {
+			return nil, err
+		}
+		ss := mm.SeriesByIDSlice(ids)
+
+		// Determine a list of keys from condition.
+		keySet, ok, err := mm.TagKeysByExpr(cond)
+		if err != nil {
+			return nil, err
+		}
+
+		// Loop over all keys for each series.
+		m := make(map[KeyValue]struct{}, len(ss))
+		for _, series := range ss {
+			for key, value := range series.Tags {
+				if !ok {
+					// nop
+				} else if _, exists := keySet[key]; !exists {
+					continue
+				}
+				m[KeyValue{key, value}] = struct{}{}
+			}
+		}
+
+		// Return an empty slice if there are no key/value matches.
+		if len(m) == 0 {
+			continue
+		}
+
+		// Sort key/value set.
+		a := make([]KeyValue, 0, len(m))
+		for kv := range m {
+			a = append(a, kv)
+		}
+		sort.Sort(KeyValues(a))
+		tagValues[i].Values = a
+	}
+
+	return tagValues, nil
+}
+
+type KeyValue struct {
+	Key, Value string
+}
+
+type KeyValues []KeyValue
+
+func (a KeyValues) Len() int      { return len(a) }
+func (a KeyValues) Swap(i, j int) { a[i], a[j] = a[j], a[i] }
+func (a KeyValues) Less(i, j int) bool {
+	ki, kj := a[i].Key, a[j].Key
+	if ki == kj {
+		return a[i].Value < a[j].Value
+	}
+	return ki < kj
+}
+
 // filterShowSeriesResult will limit the number of series returned based on the limit and the offset.
 // Unlike limit and offset on SELECT statements, the limit and offset don't apply to the number of Rows, but
 // to the number of total Values returned, since each Value represents a unique series.
@@ -718,18 +1006,6 @@ func (e *Store) filterShowSeriesResult(limit, offset int, rows models.Rows) mode
 	return filteredSeries
 }
 
-// IsRetryable returns true if this error is temporary and could be retried
-func IsRetryable(err error) bool {
-	if err == nil {
-		return true
-	}
-
-	if strings.Contains(err.Error(), "field type conflict") {
-		return false
-	}
-	return true
-}
-
 // DecodeStorePath extracts the database and retention policy names
 // from a given shard or WAL path.
 func DecodeStorePath(shardOrWALPath string) (database, retentionPolicy string) {
@@ -795,28 +1071,3 @@ func measurementsFromSourcesOrDB(db *DatabaseIndex, sources ...influxql.Source)
 
 	return measurements, nil
 }
-
-// throttle is a simple channel based concurrency limiter.  It uses a fixed
-// size channel to limit callers from proceeding until there is a value avalable
-// in the channel.  If all are in-use, the caller blocks until one is freed.
-type throttle struct {
-	c chan struct{}
-}
-
-func newthrottle(limit int) *throttle {
-	t := &throttle{
-		c: make(chan struct{}, limit),
-	}
-	for i := 0; i < limit; i++ {
-		t.c <- struct{}{}
-	}
-	return t
-}
-
-func (t *throttle) take() {
-	<-t.c
-}
-
-func (t *throttle) release() {
-	t.c <- struct{}{}
-}
